import system_tests


class IntegerOverflowInWebpImageReadMetadata(
        metaclass=system_tests.CaseMeta):
    """
    Regression test for the bug described in:
    https://github.com/Exiv2/exiv2/issues/791

    Due to an integer overflow bug (#791), this test triggers a 4GB
    memory allocation. So the test will fail with a std::bad_alloc
    exception if less than 4GB is available.  On Linux, you can use
    `ulimit -v 4000000` to reduce the available memory to slightly
    less than 4GB.
    """
    url = "https://github.com/Exiv2/exiv2/issues/791"

    filename = system_tests.path(
        "$data_path/issue_791_poc1.webp"
    )
    commands = ["$exiv2 $filename"]
    stdout = [""]
    stderr = ["""Exiv2 exception in print action for file $filename:
$kerCorruptedMetadata
"""
]
    retval = [1]



# -*- coding: utf-8 -*-

from system_tests import CaseMeta, path


class OutOfMemoryInPsdImageReadMetadata(metaclass=CaseMeta):
    """
    Regression test for the bug described in:
    https://github.com/Exiv2/exiv2/issues/855

    There is no bounds check on the value of resourceSize,
    leading to an out-of-memory error.
    """
    url = "https://github.com/Exiv2/exiv2/issues/855"

    filename = path("$data_path/issue_855_poc.psd")
    commands = ["$exiv2 -b $filename"]
    stdout = [""]
    stderr = ["""Warning: Failed to decode IPTC metadata.
Exiv2 exception in print action for file $filename:
corrupted image metadata
"""
]
    retval = [1]



# -*- coding: utf-8 -*-

from system_tests import CaseMeta, path, check_no_ASAN_UBSAN_errors

class coverage_xmp_doctype(metaclass=CaseMeta):
    """
    Test added to improve code coverage in xmpsidecar.cpp after
    Codecov complained about a lack of code coverage in this PR:
    https://github.com/Exiv2/exiv2/pull/1878
    """

    filename = path("$data_path/coverage_xmp_doctype.exv")
    commands = ["$exiv2 $filename"]
    stderr = ["""Error: XMP Toolkit error 201: Error in XMLValidator
Warning: Failed to decode XMP metadata.
$filename: No Exif data found in the file
"""]
    retval = [253]

    compare_stdout = check_no_ASAN_UBSAN_errors



# -*- coding: utf-8 -*-

from system_tests import CaseMeta, path


class CrwEncode0x1810IntegerOverflow(metaclass=CaseMeta):
    """
    Regression test for the bug described in:
    https://github.com/Exiv2/exiv2/issues/1530
    """
    url = "https://github.com/Exiv2/exiv2/issues/1530"

    filename1 = path("$data_path/issue_1530_poc.crw")
    filename2 = path("$data_path/issue_1530_poc.exv")
    commands = ["$exiv2 in $filename1 $filename2"]
    stdout = [""]
    stderr = [
"""$filename1: Could not write metadata to file: $kerCorruptedMetadata
"""]
    retval = [1]



# -*- coding: utf-8 -*-

from system_tests import CaseMeta, path


class WebPImageDoWriteMetadataOutOfBoundsRead(metaclass=CaseMeta):
    """
    Regression test for the bug described in:
    https://github.com/Exiv2/exiv2/security/advisories/GHSA-jgm9-5fw5-pw9p
    """
    url = "https://github.com/Exiv2/exiv2/security/advisories/GHSA-jgm9-5fw5-pw9p"

    filename1 = path("$data_path/issue_ghsa_jgm9_5fw5_pw9p_poc.jp2")
    filename2 = path("$data_path/issue_ghsa_jgm9_5fw5_pw9p_poc.exv")
    commands = ["$exiv2 in $filename1"]
    stdout = [""]
    stderr = [
"""Warning: Directory Image, entry 0x0000 has unknown Exif (TIFF) type 0; setting type size 1.
Warning: Directory Image, entry 0x0000 has unknown Exif (TIFF) type 0; setting type size 1.
Warning: Directory Image, entry 0x0000 has unknown Exif (TIFF) type 0; setting type size 1.
Warning: Directory Image, entry 0x0000 has unknown Exif (TIFF) type 0; setting type size 1.
Warning: Directory Image, entry 0x0000 has unknown Exif (TIFF) type 0; setting type size 1.
Warning: Directory Image, entry 0x0000 has unknown Exif (TIFF) type 0; setting type size 1.
Warning: Directory Image, entry 0x0000 has unknown Exif (TIFF) type 0; setting type size 1.
Warning: Directory Image, entry 0x0000 has unknown Exif (TIFF) type 0; setting type size 1.
Warning: Directory Image, entry 0x0000 has unknown Exif (TIFF) type 0; setting type size 1.
Warning: Directory Image, entry 0x010f has unknown Exif (TIFF) type 0; setting type size 1.
Warning: Directory Image, entry 0x0000 has unknown Exif (TIFF) type 0; setting type size 1.
Warning: Directory Image, entry 0x010f has unknown Exif (TIFF) type 0; setting type size 1.
Warning: Directory Image, entry 0x0000 has unknown Exif (TIFF) type 0; setting type size 1.
Warning: Directory Image, entry 0x0000 has unknown Exif (TIFF) type 0; setting type size 1.
Warning: Directory Image, entry 0x0000 has unknown Exif (TIFF) type 0; setting type size 1.
Warning: Directory Image, entry 0x0000 has unknown Exif (TIFF) type 0; setting type size 1.
Warning: Directory Image, entry 0x0000 has unknown Exif (TIFF) type 0; setting type size 1.
Warning: Directory Image, entry 0x0000 has unknown Exif (TIFF) type 0; setting type size 1.
Warning: Directory Image, entry 0x0000 has unknown Exif (TIFF) type 0; setting type size 1.
Warning: Directory Image, entry 0x0000 has unknown Exif (TIFF) type 0; setting type size 1.
Warning: Directory Image, entry 0x8769 has unknown Exif (TIFF) type 0; setting type size 1.
Warning: Directory Image, entry 0x8769 doesn't look like a sub-IFD.
Warning: Directory Image, entry 0x8825 doesn't look like a sub-IFD.
$filename1: Could not write metadata to file: $kerCorruptedMetadata
"""]
    retval = [1]



# -*- coding: utf-8 -*-

from system_tests import CaseMeta, path


class WebPImageDoWriteMetadataOutOfBoundsRead(metaclass=CaseMeta):
    """
    Regression test for the bug described in:
    https://github.com/Exiv2/exiv2/security/advisories/GHSA-5p8g-9xf3-gfrr
    """
    url = "https://github.com/Exiv2/exiv2/security/advisories/GHSA-5p8g-9xf3-gfrr"

    filename1 = path("$data_path/issue_ghsa_5p8g_9xf3_gfrr_poc.webp")
    filename2 = path("$data_path/issue_ghsa_5p8g_9xf3_gfrr_poc.exv")
    commands = ["$exiv2 in $filename1"]
    stdout = [""]
    stderr = [
"""Warning: Directory Nikon3, entry 0x002b has unknown Exif (TIFF) type 64002; setting type size 1.
Warning: Directory Nikon3, entry 0x002b has unknown Exif (TIFF) type 64002; setting type size 1.
$filename1: Could not write metadata to file: $kerCorruptedMetadata
"""]
    retval = [1]



# -*- coding: utf-8 -*-

from system_tests import CaseMeta, CopyFiles, path
@CopyFiles("$data_path/IMG_0246.exv")

class test_issue_1046Test(metaclass=CaseMeta):

    filename = path("$data_path/IMG_0246_copy.exv")
    # Use the """ string syntax for -M"set Bla.Dee.Bla" commands to ensure they run on Windows!
    commands  = [  "$exiv2 -pt -b -g GPSInfo -g UserCom    $filename"
                ,  "$exiv2 -pv -b -g GPSInfo -g UserCom    $filename"
                ,"""$exiv2 -M"set Exif.Photo.UserComment charset=Ascii how now brown cow" $filename"""
                ,  "$exiv2 -pt -b           -g UserComment $filename"
                ,  "$exiv2 -pv -b           -g UserComment $filename"
                ,"""$exiv2 -M"set Exif.Photo.UserComment charset=Unicode \u0041\u0042" $filename"""
                ,  "$exiv2 -pt -b           -g UserComment $filename"
                ,  "$exiv2 -pv -b           -g UserComment $filename"
                ,"""$exiv2 -M"set Exif.Photo.UserComment hello world" $filename"""
                ,  "$exiv2 -pt -b           -g UserComment $filename"
                ,  "$exiv2 -pv -b           -g UserComment $filename"
                ,"""$exiv2 -M"set Exif.GPSInfo.GPSProcessingMethod charset=Unicode \u0052\u006f\u0062\u0069\u006e" $filename"""
                ,"""$exiv2 -M"set Exif.GPSInfo.GPSAreaInformation  charset=Ascii area information"                 $filename"""
                ,  "$exiv2 -pt -b -g GPSPro -g GPSArea      $filename"
                ,  "$exiv2 -pv -b -g GPSPro -g GPSArea      $filename"
                ]
    stdout = ["""Exif.Photo.UserComment                       Undefined  12  charset=Unicode AB
Exif.GPSInfo.GPSVersionID                    Byte        4  2.2.0.0
Exif.GPSInfo.GPSLatitudeRef                  Ascii       2  North
Exif.GPSInfo.GPSLatitude                     Rational    3  51deg 23' 13"
Exif.GPSInfo.GPSLongitudeRef                 Ascii       2  West
Exif.GPSInfo.GPSLongitude                    Rational    3  0deg 44' 27"
Exif.GPSInfo.GPSAltitudeRef                  Byte        1  Above sea level
Exif.GPSInfo.GPSAltitude                     Rational    1  104.2 m
Exif.GPSInfo.GPSTimeStamp                    Rational    3  10:34:11
Exif.GPSInfo.GPSMapDatum                     Ascii       7  WGS-84
Exif.GPSInfo.GPSProcessingMethod             Undefined  18  charset=Ascii HYBRID-FIX
Exif.GPSInfo.GPSAreaInformation              Undefined  23  Surrey, England
Exif.GPSInfo.GPSDateStamp                    Ascii      20  2020:02:02 10:34:11
""","""0x9286 Photo        UserComment                 Undefined  12  charset=Unicode AB
0x0000 GPSInfo      GPSVersionID                Byte        4  2 2 0 0
0x0001 GPSInfo      GPSLatitudeRef              Ascii       2  N
0x0002 GPSInfo      GPSLatitude                 Rational    3  51/1 23/1 13/1
0x0003 GPSInfo      GPSLongitudeRef             Ascii       2  W
0x0004 GPSInfo      GPSLongitude                Rational    3  0/1 44/1 27/1
0x0005 GPSInfo      GPSAltitudeRef              Byte        1  0
0x0006 GPSInfo      GPSAltitude                 Rational    1  10420/100
0x0007 GPSInfo      GPSTimeStamp                Rational    3  10/1 34/1 11/1
0x0012 GPSInfo      GPSMapDatum                 Ascii       7  WGS-84
0x001b GPSInfo      GPSProcessingMethod         Undefined  18  charset=Ascii HYBRID-FIX
0x001c GPSInfo      GPSAreaInformation          Undefined  23  Surrey, England
0x001d GPSInfo      GPSDateStamp                Ascii      20  2020:02:02 10:34:11
""","","""Exif.Photo.UserComment                       Undefined  25  charset=Ascii how now brown cow
""","""0x9286 Photo        UserComment                 Undefined  25  charset=Ascii how now brown cow
""","","""Exif.Photo.UserComment                       Undefined  12  charset=Unicode AB
""","""0x9286 Photo        UserComment                 Undefined  12  charset=Unicode AB
""","","""Exif.Photo.UserComment                       Undefined  19  hello world
""","""0x9286 Photo        UserComment                 Undefined  19  hello world
""","","","""Exif.GPSInfo.GPSProcessingMethod             Undefined  18  charset=Unicode Robin
Exif.GPSInfo.GPSAreaInformation              Undefined  24  charset=Ascii area information
""","""0x001b GPSInfo      GPSProcessingMethod         Undefined  18  charset=Unicode Robin
0x001c GPSInfo      GPSAreaInformation          Undefined  24  charset=Ascii area information
"""
             ]
    stderr = [""]*len(commands)
    retval = [ 0]*len(commands)



# -*- coding: utf-8 -*-

from system_tests import CaseMeta, path
import unittest

@unittest.skip("Skipping test using option -pR (only for Debug mode)")
class SegmentationFaultPngImage(metaclass=CaseMeta):
    """
    Regression test for the bug described in:
    https://github.com/Exiv2/exiv2/issues/828
    """
    url = "https://github.com/Exiv2/exiv2/issues/828"

    filename = path("$data_path/issue_828_poc.png")
    commands = ["$exiv2 -pR $filename"]
    stdout = ["""STRUCTURE OF PNG FILE: $filename
 address | chunk |  length | data                           | checksum
       8 | IHDR  |      13 | .............                  | 0x085f152d
      33 | gAMA  |       4 | ....                           | 0x0bfc6105
      49 | PLTE  |     717 | .........].....Y..q..}..q..y.. | 0x7b35a4a5
     778 | tRNS  |     238 | -U...............}...=..e..... | 0xebd29012
    1028 | bKGD  |       1 | .                              | 0xcfb7d237
    1041 | tEXt  |      14 | Title.IceAlpha                 | 0xc562f8cb
    1067 | tEXt  |      31 | Author.Pieter S. van der Meule | 0x6bfe26a8
    1110 | tEXt  |     411 | Description.The original image | 0xe12effe4
"""
]
    stderr = [
        """$exiv2_exception_message $filename:
$kerCorruptedMetadata
"""]
    retval = [1]



# -*- coding: utf-8 -*-

import system_tests


class ThrowsWhenColorProfileInTiffMetadataIsCorrupted(metaclass=system_tests.CaseMeta):

    url = "https://github.com/Exiv2/exiv2/issues/457"

    filename = "$data_path/issue_457_poc"

    commands = ["$exiv2 " + filename]
    compare_stderr = system_tests.check_no_ASAN_UBSAN_errors
    retval = [1]



# -*- coding: utf-8 -*-

import system_tests


class DecodeIHDRChunkOutOfBoundsRead(metaclass=system_tests.CaseMeta):

    url = "https://github.com/Exiv2/exiv2/issues/170"

    filename = "$data_path/issue_170_poc"

    commands = ["$exiv2 " + filename]
    stdout = [""]
    stderr = ["""$exiv2_exception_message """ + filename + """:
$kerFailedToReadImageData
"""]
    retval = [1]



# -*- coding: utf-8 -*-

import system_tests

class CheckMWG_KW_Schema(metaclass=system_tests.CaseMeta):

    url = "http://dev.exiv2.org/issues/1040"

    filename = system_tests.path("$data_path/exiv2-bug1040.jpg")
    commands = [ "$exiv2 -px $filename" ]

    stdout = [ """Xmp.mwg-kw.Keywords                          XmpText     0  type="Struct"
Xmp.mwg-kw.Keywords/mwg-kw:Hierarchy         XmpText     0  type="Bag"
Xmp.mwg-kw.Keywords/mwg-kw:Hierarchy[1]      XmpText     0  type="Struct"
Xmp.mwg-kw.Keywords/mwg-kw:Hierarchy[1]/mwg-kw:Keyword XmpText     3  cat
Xmp.mwg-kw.Keywords/mwg-kw:Hierarchy[1]/mwg-kw:Children XmpText     0  type="Bag"
Xmp.mwg-kw.Keywords/mwg-kw:Hierarchy[1]/mwg-kw:Children[1] XmpText     0  type="Struct"
Xmp.mwg-kw.Keywords/mwg-kw:Hierarchy[1]/mwg-kw:Children[1]/mwg-kw:Keyword XmpText     7  Siamese
Xmp.mwg-kw.Keywords/mwg-kw:Hierarchy[2]      XmpText     0  type="Struct"
Xmp.mwg-kw.Keywords/mwg-kw:Hierarchy[2]/mwg-kw:Keyword XmpText     7  Felidae
Xmp.mwg-kw.Keywords/mwg-kw:Hierarchy[2]/mwg-kw:Children XmpText     0  type="Bag"
Xmp.mwg-kw.Keywords/mwg-kw:Hierarchy[2]/mwg-kw:Children[1] XmpText     0  type="Struct"
Xmp.mwg-kw.Keywords/mwg-kw:Hierarchy[2]/mwg-kw:Children[1]/mwg-kw:Keyword XmpText     6  Jaguar
"""
    ]
    stderr = [""]
    retval = [0]



# -*- coding: utf-8 -*-

from system_tests import CaseMeta, CopyTmpFiles, path
@CopyTmpFiles("$data_path/test_issue_1180.exv")

class test_issue_1180Test(metaclass=CaseMeta):

    filename  = path("$tmp_path/test_issue_1180.exv")
    dash_t    = path("$tmp_path/20200424_174415.exv")      # -t renames file
    dash_T    = path("$tmp_path/20200424_154415.exv")      # -T renames file
    commands  = [  "$exiv2 -K Exif.Image.DateTime       $filename"
                ,  "$exiv2 -t          --force          $filename"
                ,  "$exiv2 -K Exif.Image.DateTime       $dash_t"
                ,  "$exiv2 -a -02:00   --force          $dash_t"
                ,  "$exiv2 -K Exif.Image.DateTime       $dash_t"
                ,  "$exiv2 -t          --force          $dash_t"
                ,  "$exiv2 -K Exif.Image.DateTime       $dash_T"
                ]
    stdout   =  ["Exif.Image.DateTime                          Ascii      50  2020-04-24 17:44:15 \n"
                ,""
                ,"Exif.Image.DateTime                          Ascii      50  2020-04-24 17:44:15 \n"
                ,""
                ,"Exif.Image.DateTime                          Ascii      20  2020:04:24 15:44:15\n"
                ,""
                ,"Exif.Image.DateTime                          Ascii      20  2020:04:24 15:44:15\n"
                ]
    stderr = [""]*len(commands)
    retval = [ 0]*len(commands)



# -*- coding: utf-8 -*-

import system_tests


class CheckTimeZones(metaclass=system_tests.CaseMeta):

    url = "http://dev.exiv2.org/issues/1231"
    filenames = ["$data_path/exiv2-bug1231a.jpg",
                 "$data_path/exiv2-bug1231b.jpg"]

    commands = [
        "$exiv2 -pa --grep CanonTi/i " + filenames[0],
        "$exiv2 -pa --grep CanonTi/i " + filenames[1]
    ]

    stdout = [
        """Exif.CanonTi.TimeZone                        SLong       1  60
Exif.CanonTi.TimeZoneCity                    SLong       1  n/a
Exif.CanonTi.DaylightSavings                 SLong       1  0
""",
        """Exif.CanonTi.TimeZone                        SLong       1  120
Exif.CanonTi.TimeZoneCity                    SLong       1  n/a
Exif.CanonTi.DaylightSavings                 SLong       1  60
""",
    ]

    stderr = [""] * len(stdout)
    retval = [0] * len(stdout)



# -*- coding: utf-8 -*-

import system_tests


class NikonLens(metaclass=system_tests.CaseMeta):

    filenames = ["$data_path/exiv2-bug1014.exv", "$data_path/exiv2-bug1014_2.exv"]
    commands = ["$exiv2 -pa --grep lens/i %s" % filenames[0], "$exiv2 -pa --grep lens/i %s" % filenames[1]]

    stdout = ["""Exif.Nikon3.LensType                         Byte        1  D G VR
Exif.Nikon3.Lens                             Rational    4  24-70mm F2.8
Exif.Nikon3.LensFStops                       Undefined   4  6
Exif.NikonLd3.LensIDNumber                   Byte        1  Nikon AF-S Nikkor 24-70mm f/2.8E ED VR
Exif.NikonLd3.LensFStops                     Byte        1  F6.0
""",
              """Exif.Nikon3.LensType                         Byte        1  D G VR
Exif.Nikon3.Lens                             Rational    4  70-200mm F2.8
Exif.Nikon3.LensFStops                       Undefined   4  6
Exif.NikonLd3.LensIDNumber                   Byte        1  Nikon AF-S Nikkor 70-200mm f/2.8E FL ED VR
Exif.NikonLd3.LensFStops                     Byte        1  F6.0
"""
              ]
    stderr = ["", ""]
    retval = [0, 0]



# -*- coding: utf-8 -*-

import system_tests


@system_tests.CopyFiles("$data_path/mini9.tif")
class TestTiffTestProg(metaclass=system_tests.CaseMeta):

    commands = ["$tiff_test $data_path/mini9_copy.tif"]

    stdout = [
        """Test 1: Writing empty Exif data without original binary data: ok.
Test 2: Writing empty Exif data with original binary data: ok.
Test 3: Wrote non-empty Exif data without original binary data:
Exif.Image.ExifTag                           0x8769 Long        1  26
Exif.Photo.DateTimeOriginal                  0x9003 Ascii      18  Yesterday at noon
MIME type:  image/tiff
Image size: 9 x 9
Before
Exif.Image.NewSubfileType                    0x00fe Long        1  0
Exif.Image.ImageWidth                        0x0100 Short       1  9
Exif.Image.ImageLength                       0x0101 Short       1  9
Exif.Image.BitsPerSample                     0x0102 Short       3  8 8 8
Exif.Image.Compression                       0x0103 Short       1  1
Exif.Image.PhotometricInterpretation         0x0106 Short       1  2
Exif.Image.DocumentName                      0x010d Ascii      24  /home/ahuggel/mini9.tif
Exif.Image.ImageDescription                  0x010e Ascii      18  Created with GIMP
Exif.Image.StripOffsets                      0x0111 Long        1  8
Exif.Image.Orientation                       0x0112 Short       1  1
Exif.Image.SamplesPerPixel                   0x0115 Short       1  3
Exif.Image.RowsPerStrip                      0x0116 Short       1  64
Exif.Image.StripByteCounts                   0x0117 Long        1  243
Exif.Image.XResolution                       0x011a Rational    1  1207959552/16777216
Exif.Image.YResolution                       0x011b Rational    1  1207959552/16777216
Exif.Image.PlanarConfiguration               0x011c Short       1  1
Exif.Image.ResolutionUnit                    0x0128 Short       1  2
======
After
Exif.Image.NewSubfileType                    0x00fe Long        1  0
Exif.Image.ImageWidth                        0x0100 Short       1  9
Exif.Image.ImageLength                       0x0101 Short       1  9
Exif.Image.BitsPerSample                     0x0102 Short       3  8 8 8
Exif.Image.Compression                       0x0103 Short       1  1
Exif.Image.PhotometricInterpretation         0x0106 Short       1  2
Exif.Image.DocumentName                      0x010d Ascii      24  /home/ahuggel/mini9.tif
Exif.Image.ImageDescription                  0x010e Ascii      18  Created with GIMP
Exif.Image.StripOffsets                      0x0111 Long        1  8
Exif.Image.Orientation                       0x0112 Short       1  1
Exif.Image.SamplesPerPixel                   0x0115 Short       1  3
Exif.Image.RowsPerStrip                      0x0116 Short       1  64
Exif.Image.StripByteCounts                   0x0117 Long        1  243
Exif.Image.XResolution                       0x011a Rational    1  1207959552/16777216
Exif.Image.YResolution                       0x011b Rational    1  1207959552/16777216
Exif.Image.PlanarConfiguration               0x011c Short       1  1
Exif.Image.ResolutionUnit                    0x0128 Short       1  2
Exif.Photo.DateTimeOriginal                  0x9003 Ascii      18  Yesterday at noon
"""
    ]
    stderr = [""]
    retval = [0]



# -*- coding: utf-8 -*-

import system_tests
class issue_1522_exif_asan(metaclass=system_tests.CaseMeta):
    url      = "https://github.com/Exiv2/exiv2/issues/1522"
    filename = "$data_path/poc_1522.jp2"
    commands = ["$exiv2 -q  $filename"
               ,"$exiv2 -pS $filename"
               ]
    retval   = [ 1,1 ]
    stderr   = [ """$exiv2_exception_message $filename:
$kerCorruptedMetadata
""","""$exiv2_exception_message $filename:
$kerCorruptedMetadata
"""] 
    stdout   = ["","""STRUCTURE OF JPEG2000 FILE: $filename
 address |   length | box       | data
       0 |       12 | jP        | 
      12 |       25 | uuid      | Exif: .
"""
]



# -*- coding: utf-8 -*-

from system_tests import CaseMeta, path
import unittest

# Issue #1821 is a memory leak caused by buggy third-party
# code in the xmpsdk sub-directory, so it isn't easy for us to fix.
# This test will fail when run with an ASAN build.
@unittest.skip("Skipping test until #1821 is fixed")

class MemoryLeakAddBinding(metaclass=CaseMeta):
    """
    Regression test for the bug described in:
    https://github.com/Exiv2/exiv2/issues/1821
    """
    url = "https://github.com/Exiv2/exiv2/issues/1821"

    filename = path("$data_path/issue_1821_poc.xmp")
    commands = ["$exiv2 -q $filename"]
    stdout = ["""File name       : $filename
File size       : 163 Bytes
MIME type       : application/rdf+xml
Image size      : 0 x 0
"""]
    stderr = ["""$filename: No Exif data found in the file
"""]
    retval = [253]



# -*- coding: utf-8 -*-

import system_tests
import unittest

# test needs system_tests.BT.vv['enable_bmff']=1
bSkip=system_tests.BT.verbose_version().get('enable_bmff')!='1'
if bSkip:
    raise unittest.SkipTest('*** requires enable_bmff=1 ***')

class BmffImageboxHandlerLargeAllocation(metaclass=system_tests.CaseMeta):
    """
    Regression test for the bug described in:
    https://github.com/Exiv2/exiv2/issues/1793
    """
    url = "https://github.com/Exiv2/exiv2/issues/1793"
    filename = "$data_path/issue_1793_poc.heic"

    if bSkip:
        commands=[]
        retval=[]
        stdin=[]
        stderr=[]
        stdout=[]
        print("*** test skipped.  requires enable_bmff=1***")
    else:
        commands = ["$exiv2 $filename"]
        stdout = [""]
        stderr = ["""Exiv2 exception in print action for file $filename:
$kerCorruptedMetadata
"""]
        retval = [1]



# -*- coding: utf-8 -*-

from system_tests import CaseMeta, CopyTmpFiles, path
@CopyTmpFiles("$data_path/Stonehenge.exv")

class test_issue_1473(metaclass=CaseMeta):
    url       = "https://github.com/Exiv2/exiv2/issues/1473"
    filename  = path("$tmp_path/Stonehenge.exv")
    commands  = [ "$exiv2 -g shown/i                              $filename" 
                , "$exiv2 -M\"set Xmp.iptc.LocationShown Lost\" $filename"
                , "$exiv2 -g shown/i                              $filename"
                ]
    stdout   =  ["","","""Xmp.iptc.LocationShown                       XmpText     4  Lost
"""]
    stderr = [""]*len(commands)
    retval = [ 1,0,0]



# -*- coding: utf-8 -*-

import os
import shutil

import system_tests


class PercentABrokenInRename(metaclass=system_tests.CaseMeta):

    url = "http://dev.exiv2.org/issues/683"

    original_file = "$data_path/exiv2-nikon-d70.jpg"
    filename = "$data_path/exiv2-nikon-d70_copy.jpg"
    new_file = "$data_path/2004-03-30-Tue-090.jpg"

    commands = ["$exiv2 -u -f -r %Y-%m-%d-%a-%j $filename"]
    stdout = [""]
    stderr = [""]
    retval = [0]

    def setUp(self):
        shutil.copyfile(self.original_file, self.filename)

    def tearDown(self):
        self.assertTrue(os.path.exists(self.new_file))
        os.remove(self.new_file)



# -*- coding: utf-8 -*-

from system_tests import CaseMeta, path

class test_issue_990Test(metaclass=CaseMeta):

    filename = path("$data_path/test_issue_990.exv")
    commands  = ["$exiv2 -pa --grep lens/i $filename"
                ]
    stdout = ["""Exif.Nikon3.LensType                         Byte        1  D G VR
Exif.Nikon3.Lens                             Rational    4  70-200mm F2.8
Exif.Nikon3.LensFStops                       Undefined   4  7
Exif.NikonLd3.LensIDNumber                   Byte        1  Tamron SP 70-200mm F/2.8 Di VC USD
Exif.NikonLd3.LensFStops                     Byte        1  F7.0
"""
             ]
    stderr = [""]
    retval = [ 0]



# -*- coding: utf-8 -*-

from system_tests import CaseMeta, CopyFiles, path


URL = "http://dev.exiv2.org/issues/937"


class DarwinCoreXmpMetadataPrint(metaclass=CaseMeta):

    filename = path("$data_path/exiv2-bug937.jpg")

    commands = [
         "$exiv2 -q -pa      -b -g dwc  $filename",
         "$exiv2 -q -PXkyctl -b -g Date $filename",
    ]

    stdout = [
        """Xmp.dwc.Record                               XmpText     0  type="Struct"
Xmp.dwc.Record/dwc:institutionID             XmpText    25  Charles Darwin Foundation
Xmp.dwc.Record/dwc:collectionID              XmpText    29  urn:lsid:biocol.org:col:34818
Xmp.dwc.Record/dwc:institutionCode           XmpText     3  CDS
Xmp.dwc.Record/dwc:datasetID                 XmpText     3  MVZ
Xmp.dwc.Record/dwc:collectionCode            XmpText     7  Mammals
Xmp.dwc.Record/dwc:datasetName               XmpText    25  Grinnell Resurvey Mammals
Xmp.dwc.Record/dwc:ownerInstitutionCode      XmpText     3  NPS
Xmp.dwc.Record/dwc:basisOfRecord             XmpText    14  FossilSpecimen
Xmp.dwc.Record/dwc:informationWithheld       XmpText    53  location information not given for endangered species
Xmp.dwc.Record/dwc:dataGeneralizations       XmpText    90  Coordinates generalized from original GPS coordinates to the nearest half degree grid cell
Xmp.dwc.Record/dwc:dynamicProperties         XmpText    45  targusLengthInMeters=0.014; weightInGrams=120
Xmp.dwc.Occurrence                           XmpText     0  type="Struct"
Xmp.dwc.Occurrence/dwc:occurrenceID          XmpText    28  urn:lsid:nhm.ku.edu:Herps:32
Xmp.dwc.Occurrence/dwc:catalogNumber         XmpText     9  2008.1334
Xmp.dwc.Occurrence/dwc:occurrenceDetails     XmpText    50  http://mvzarctos.berkeley.edu/guid/MVZ:Mamm:165861
Xmp.dwc.Occurrence/dwc:recordNumber          XmpText     8  OPP 7101
Xmp.dwc.Occurrence/dwc:recordedBy            XmpText    17  Oliver P. Pearson
Xmp.dwc.Occurrence/dwc:individualID          XmpText    10  U.amer. 44
Xmp.dwc.Occurrence/dwc:individualCount       XmpText     1  1
Xmp.dwc.Occurrence/dwc:sex                   XmpText     6  female
Xmp.dwc.Occurrence/dwc:lifeStage             XmpText     3  egg
Xmp.dwc.Occurrence/dwc:reproductiveCondition XmpText    16  non-reproductive
Xmp.dwc.Occurrence/dwc:behavior              XmpText     8  roosting
Xmp.dwc.Occurrence/dwc:establishmentMeans    XmpText    10  cultivated
Xmp.dwc.Occurrence/dwc:occurrenceStatus      XmpText     7  present
Xmp.dwc.Occurrence/dwc:preparations          XmpText     4  skin
Xmp.dwc.Occurrence/dwc:disposition           XmpText    13  in collection
Xmp.dwc.Occurrence/dwc:otherCatalogNumbers   XmpText    16  FNMH:Mammal:1234
Xmp.dwc.Occurrence/dwc:previousIdentifications XmpText   109  Anthus sp., field ID by G. Iglesias; Anthus correndera, expert ID by C. Cicero 2009-02-12 based on morphology
Xmp.dwc.Occurrence/dwc:associatedMedia       XmpText    71  http://arctos.database.museum/SpecimenImages/UAMObs/Mamm/2/P7291179.JPG
Xmp.dwc.Occurrence/dwc:associatedReferences  XmpText   153  Christopher J. Conroy, Jennifer L. Neuwald. 2008. Phylogeographic study of the California vole, Microtus californicus Journal of Mammalogy, 89(3):755-767
Xmp.dwc.Occurrence/dwc:associatedOccurrences XmpText    56  sibling of FMNH:Mammal:1234; sibling of FMNH:Mammal:1235
Xmp.dwc.Occurrence/dwc:associatedSequences   XmpText    16  GenBank:U34853.1
Xmp.dwc.Occurrence/dwc:associatedTaxa        XmpText    17  host:Quercus alba
Xmp.dwc.Occurrence/dwc:occurrenceRemarks     XmpText    18  found dead on road
Xmp.dwc.Event                                XmpText     0  type="Struct"
Xmp.dwc.Event/dwc:eventID                    XmpText     4  1234
Xmp.dwc.Event/dwc:samplingProtocol           XmpText   134  Penguins from space: faecal stains reveal the location of emperor penguin colonies, http://dx.doi.org/10.1111/j.1466-8238.2009.00467.x
Xmp.dwc.Event/dwc:samplingEffort             XmpText    14  40 trap-nights
Xmp.dwc.Event/dwc:earliestDate               XmpText    25  2012-09-03T00:00:00-06:00
Xmp.dwc.Event/dwc:latestDate                 XmpText    25  2013-01-27T00:00:00-06:00
Xmp.dwc.Event/dwc:eventTime                  XmpText    25  1899-12-30T01:41:33+03:00
Xmp.dwc.Event/dwc:startDayOfYear             XmpText     1  1
Xmp.dwc.Event/dwc:endDayOfYear               XmpText     3  366
Xmp.dwc.Event/dwc:year                       XmpText     4  2008
Xmp.dwc.Event/dwc:month                      XmpText     2  10
Xmp.dwc.Event/dwc:day                        XmpText     2  31
Xmp.dwc.Event/dwc:habitat                    XmpText    11  oak savanna
Xmp.dwc.Event/dwc:verbatimEventDate          XmpText    11  spring 1910
Xmp.dwc.Event/dwc:fieldNumber                XmpText    16  RV Sol  87-03-08
Xmp.dwc.Event/dwc:fieldNotes                 XmpText    42  notes available in Grinnell-Miller Library
Xmp.dwc.Event/dwc:eventRemarks               LangAlt     1  lang="x-default" after the recent rains the river is nearly at flood stage
Xmp.dwc.dctermsLocation                      XmpText     0  type="Struct"
Xmp.dwc.dctermsLocation/dwc:locationID       XmpText     4  1234
Xmp.dwc.dctermsLocation/dwc:higherGeographyID XmpText    11  TGN:1002002
Xmp.dwc.dctermsLocation/dwc:higherGeography  XmpText    24  South America; Argenitna
Xmp.dwc.dctermsLocation/dwc:continent        XmpText    10  Antarctica
Xmp.dwc.dctermsLocation/dwc:waterBody        XmpText    12  Indian Ocean
Xmp.dwc.dctermsLocation/dwc:islandGroup      XmpText    21  Alexander Archipelago
Xmp.dwc.dctermsLocation/dwc:island           XmpText    13  Isla Victoria
Xmp.dwc.dctermsLocation/dwc:country          XmpText     7  Denmark
Xmp.dwc.dctermsLocation/dwc:countryCode      XmpText     2  AR
Xmp.dwc.dctermsLocation/dwc:stateProvince    XmpText     7  Montana
Xmp.dwc.dctermsLocation/dwc:county           XmpText     8  Missoula
Xmp.dwc.dctermsLocation/dwc:municipality     XmpText    10  Holzminden
Xmp.dwc.dctermsLocation/dwc:locality         XmpText    50  25 km NNEoc  Bariloche along National Road no. 237
Xmp.dwc.dctermsLocation/dwc:verbatimLocality XmpText    35  25 km NNE Bariloche por R. Nac. 237
Xmp.dwc.dctermsLocation/dwc:verbatimElevation XmpText     9  100-200 m
Xmp.dwc.dctermsLocation/dwc:minimumElevationInMeters XmpText     3  100
Xmp.dwc.dctermsLocation/dwc:maximumElevationInMeters XmpText     3  200
Xmp.dwc.dctermsLocation/dwc:verbatimDepth    XmpText     9  100-200 m
Xmp.dwc.dctermsLocation/dwc:minimumDepthInMeters XmpText     3  100
Xmp.dwc.dctermsLocation/dwc:maximumDepthInMeters XmpText     3  100
Xmp.dwc.dctermsLocation/dwc:minimumDistanceAboveSurfaceInMeters XmpText    82  1.5 meter sediment core from the bottom of a lake (at depth 20m) at 300m elevation
Xmp.dwc.dctermsLocation/dwc:maximumDistanceAboveSurfaceInMeters XmpText    82  1.5 meter sediment core from the bottom of a lake (at depth 20m) at 300m elevation
Xmp.dwc.dctermsLocation/dwc:locationAccordingTo XmpText    35  Getty Thesaurus of Geographic Names
Xmp.dwc.dctermsLocation/dwc:locationRemarks  XmpText    22  under water since 2005
Xmp.dwc.dctermsLocation/dwc:verbatimCoordinates XmpText    20  41 05 54S 121 05 34W
Xmp.dwc.dctermsLocation/dwc:verbatimLatitude XmpText    12  41 05 54.03S
Xmp.dwc.dctermsLocation/dwc:verbatimLongitude XmpText    10  121 05 34W
Xmp.dwc.dctermsLocation/dwc:verbatimCoordinateSystem XmpText     3  UTM
Xmp.dwc.dctermsLocation/dwc:verbatimSRS      XmpText     5  WGS84
Xmp.dwc.dctermsLocation/dwc:decimalLatitude  XmpText    11  -41.0983423
Xmp.dwc.dctermsLocation/dwc:decimalLongitude XmpText    12  -121.1761111
Xmp.dwc.dctermsLocation/dwc:geodeticDatum    XmpText     5  WGS84
Xmp.dwc.dctermsLocation/dwc:coordinateUncertaintyInMeters XmpText     2  30
Xmp.dwc.dctermsLocation/dwc:coordinatePrecision XmpText     7  0.00001
Xmp.dwc.dctermsLocation/dwc:pointRadiusSpatialFit XmpText     1  1
Xmp.dwc.dctermsLocation/dwc:footprintWKT     XmpText    45  POLYGON ((10 20, 11 20, 11 21, 10 21, 10 20))
Xmp.dwc.dctermsLocation/dwc:footprintSRS     XmpText    12  GCS_WGS_1984
Xmp.dwc.dctermsLocation/dwc:footprintSpatialFit XmpText     1  1
Xmp.dwc.dctermsLocation/dwc:georeferencedBy  XmpText    17  Kristina Yamamoto
Xmp.dwc.dctermsLocation/dwc:georeferenceProtocol XmpText    45  MaNIS/HerpNet/ORNIS Georeferencing Guidelines
Xmp.dwc.dctermsLocation/dwc:georeferenceSources XmpText    35  USGS 1:24,000 Florence Montana Quad
Xmp.dwc.dctermsLocation/dwc:georeferenceVerificationStatus XmpText    21  requires verification
Xmp.dwc.dctermsLocation/dwc:georeferenceRemarks XmpText    35  assumed distance by road (Hwy. 101)
Xmp.dwc.GeologicalContext                    XmpText     0  type="Struct"
Xmp.dwc.GeologicalContext/dwc:geologicalContextID XmpText     4  1234
Xmp.dwc.GeologicalContext/dwc:earliestEonOrLowestEonothem XmpText    11  Phanerozoic
Xmp.dwc.GeologicalContext/dwc:latestEonOrHighestEonothem XmpText    11  Proterozoic
Xmp.dwc.GeologicalContext/dwc:earliestEraOrLowestErathem XmpText     8  Cenozoic
Xmp.dwc.GeologicalContext/dwc:earliestPeriodOrLowestSystem XmpText     7  Neogene
Xmp.dwc.GeologicalContext/dwc:latestPeriodOrHighestSystem XmpText     8  Tertiary
Xmp.dwc.GeologicalContext/dwc:earliestEpochOrLowestSeries XmpText    11  Pleistocene
Xmp.dwc.GeologicalContext/dwc:latestEpochOrHighestSeries XmpText    14  Ibexian Series
Xmp.dwc.GeologicalContext/dwc:earliestAgeOrLowestStage XmpText     8  Atlantic
Xmp.dwc.GeologicalContext/dwc:latestAgeOrHighestStage XmpText     6  Boreal
Xmp.dwc.GeologicalContext/dwc:lowestBiostratigraphicZone XmpText     6  Boreal
Xmp.dwc.GeologicalContext/dwc:highestBiostratigraphicZone XmpText     8  Atlantic
Xmp.dwc.GeologicalContext/dwc:lithostratigraphicTerms XmpText    10  Supergroup
Xmp.dwc.GeologicalContext/dwc:group          XmpText    16  Notch Peak Group
Xmp.dwc.GeologicalContext/dwc:formation      XmpText    20  Notch Peak Formation
Xmp.dwc.GeologicalContext/dwc:member         XmpText    15  Lava Dam Member
Xmp.dwc.GeologicalContext/dwc:bed            XmpText    12  Lava Dam Bed
Xmp.dwc.Identification                       XmpText     0  type="Struct"
Xmp.dwc.Identification/dwc:identificationID  XmpText     4  1234
Xmp.dwc.Identification/dwc:identifiedBy      XmpText    15  James L. Patton
Xmp.dwc.Identification/dwc:dateIdentified    XmpText    25  2013-01-27T00:00:00-06:00
Xmp.dwc.Identification/dwc:identificationReferences XmpText    50  Aves del Noroeste Patagonico. Christie et al. 2004
Xmp.dwc.Identification/dwc:identificationRemarks XmpText   106  Distinguished between Anthus correndera and Anthus hellmayri based on the comparative lengths of the uñas
Xmp.dwc.Identification/dwc:identificationQualifier XmpText    29  aff. agrifolia var. oxyadenia
Xmp.dwc.Identification/dwc:typeStatus        XmpText   101  holotype of Ctenomys sociabilis. Pearson O. P., and M. I. Christie. 1985. Historia Natural, 5(37):388
Xmp.dwc.Taxon                                XmpText     0  type="Struct"
Xmp.dwc.Taxon/dwc:taxonID                    XmpText    36  8fa58e08-08de-4ac1-b69c-1235340b7001
Xmp.dwc.Taxon/dwc:scientificNameID           XmpText    35  urn:lsid:ipni.org:names:37829-1:1.3
Xmp.dwc.Taxon/dwc:acceptedNameUsageID        XmpText    36  8fa58e08-08de-4ac1-b69c-1235340b7001
Xmp.dwc.Taxon/dwc:parentNameUsageID          XmpText    36  8fa58e08-08de-4ac1-b69c-1235340b7001
Xmp.dwc.Taxon/dwc:nameAccordingToID          XmpText    33  doi:10.1016/S0269-915X(97)80026-2
Xmp.dwc.Taxon/dwc:namePublishedInID          XmpText    29  http://hdl.handle.net/10199/7
Xmp.dwc.Taxon/dwc:taxonConceptID             XmpText    36  8fa58e08-08de-4ac1-b69c-1235340b7001
Xmp.dwc.Taxon/dwc:scientificName             XmpText    19  Ctenomys sociabilis
Xmp.dwc.Taxon/dwc:acceptedNameUsage          XmpText    46  Tamias minimus valid name for Eutamias minimus
Xmp.dwc.Taxon/dwc:parentNameUsage            XmpText     9  Rubiaceae
Xmp.dwc.Taxon/dwc:originalNameUsage          XmpText    36  Gasterosteus saltatrix Linnaeus 1768
Xmp.dwc.Taxon/dwc:nameAccordingTo            XmpText   235  McCranie, J. R., D. B. Wake, and L. D. Wilson. 1996. The taxonomic status of Bolitoglossa schmidti, with comments on the biology of the Mesoamerican salamander Bolitoglossa dofleini (Caudata: Plethodontidae). Carib. J. Sci. 32:395-398.
Xmp.dwc.Taxon/dwc:namePublishedIn            XmpText    68  Pearson O. P., and M. I. Christie. 1985. Historia Natural, 5(37):388
Xmp.dwc.Taxon/dwc:higherClassification       XmpText   124  Animalia;Chordata;Vertebrata;Mammalia;Theria;Eutheria;Rodentia;Hystricognatha;Hystricognathi;Ctenomyidae;Ctenomyini;Ctenomys
Xmp.dwc.Taxon/dwc:kingdom                    XmpText     8  Animalia
Xmp.dwc.Taxon/dwc:phylum                     XmpText     8  Chordata
Xmp.dwc.Taxon/dwc:class                      XmpText    10  Vertebrata
Xmp.dwc.Taxon/dwc:order                      XmpText     8  Mammalia
Xmp.dwc.Taxon/dwc:family                     XmpText     7  Felidae
Xmp.dwc.Taxon/dwc:genus                      XmpText     4  Puma
Xmp.dwc.Taxon/dwc:subgenus                   XmpText     4  Puma
Xmp.dwc.Taxon/dwc:specificEpithet            XmpText     8  concolor
Xmp.dwc.Taxon/dwc:taxonRank                  XmpText    10  subspecies
Xmp.dwc.Taxon/dwc:verbatimTaxonRank          XmpText    12  Agamospecies
Xmp.dwc.Taxon/dwc:infraspecificEpithet       XmpText     8  concolor
Xmp.dwc.Taxon/dwc:scientificNameAuthorship   XmpText    19  (Torr.) J.T. Howell
Xmp.dwc.Taxon/dwc:nomenclaturalCode          XmpText     4  ICBN
Xmp.dwc.Taxon/dwc:vernacularName             LangAlt     1  lang="x-default" Andean Condor
Xmp.dwc.Taxon/dwc:taxonomicStatus            XmpText     7  invalid
Xmp.dwc.Taxon/dwc:nomenclaturalStatus        XmpText    11  nom. ambig.
Xmp.dwc.Taxon/dwc:taxonRemarks               XmpText    39  this name is a mispelling in common use
Xmp.dwc.ResourceRelationship                 XmpText     0  type="Struct"
Xmp.dwc.ResourceRelationship/dwc:resourceRelationshipID XmpText     4  1234
Xmp.dwc.ResourceRelationship/dwc:resourceID  XmpText     4  1234
Xmp.dwc.ResourceRelationship/dwc:relatedResourceID XmpText     4  1234
Xmp.dwc.ResourceRelationship/dwc:relationshipOfResource XmpText    12  duplicate of
Xmp.dwc.ResourceRelationship/dwc:relationshipAccordingTo XmpText    14  Julie Woodruff
Xmp.dwc.ResourceRelationship/dwc:relationshipEstablishedDate XmpText    25  2013-01-27T00:00:00-06:00
Xmp.dwc.ResourceRelationship/dwc:relationshipRemarks XmpText    30  pollinator captured in the act
Xmp.dwc.MeasurementOrFact                    XmpText     0  type="Struct"
Xmp.dwc.MeasurementOrFact/dwc:measurementID  XmpText     4  1234
Xmp.dwc.MeasurementOrFact/dwc:measurementType XmpText    11  tail length
Xmp.dwc.MeasurementOrFact/dwc:measurementValue XmpText     2  45
Xmp.dwc.MeasurementOrFact/dwc:measurementAccuracy XmpText     4  0.01
Xmp.dwc.MeasurementOrFact/dwc:measurementUnit XmpText     2  mm
Xmp.dwc.MeasurementOrFact/dwc:measurementDeterminedDate XmpText    25  2013-01-27T00:00:00-06:00
Xmp.dwc.MeasurementOrFact/dwc:measurementDeterminedBy XmpText    18  Javier de la Torre
Xmp.dwc.MeasurementOrFact/dwc:measurementMethod XmpText    20  barometric altimeter
Xmp.dwc.MeasurementOrFact/dwc:measurementRemarks XmpText    19  tip of tail missing
""",
        """Xmp.exif.DateTimeDigitized                   Date and Time Digitized        XmpText    29  2008-03-14T11:31:48.098-07:00
Xmp.exif.DateTimeOriginal                    Date and Time Original         XmpText    25  2008-03-14T13:59:26-06:00
Xmp.photoshop.DateCreated                    Date Created                   XmpText    29  2008-03-14T13:59:26.054-06:00
Xmp.xmp.MetadataDate                         Metadata Date                  XmpText    29  2013-02-07T21:56:33.820-06:00
Xmp.xmp.CreateDate                           Create Date                    XmpText    24  2008-03-14T20:59:26.535Z
Xmp.xmp.ModifyDate                           Modify Date                    XmpText    25  2013-01-27T14:02:29-06:00
Xmp.dwc.Event/dwc:earliestDate               Event/dwc:earliestDate         XmpText    25  2012-09-03T00:00:00-06:00
Xmp.dwc.Event/dwc:latestDate                 Event/dwc:latestDate           XmpText    25  2013-01-27T00:00:00-06:00
Xmp.dwc.Event/dwc:verbatimEventDate          Event/dwc:verbatimEventDate    XmpText    11  spring 1910
Xmp.dwc.ResourceRelationship/dwc:relationshipEstablishedDate ResourceRelationship/dwc:relationshipEstablishedDate XmpText    25  2013-01-27T00:00:00-06:00
Xmp.dwc.MeasurementOrFact/dwc:measurementDeterminedDate MeasurementOrFact/dwc:measurementDeterminedDate XmpText    25  2013-01-27T00:00:00-06:00
"""
    ]

    stderr = [""] * 2

    retval = [0] * 2


@CopyFiles("$data_path/exiv2.dc.dwc.i18n.jpg")
class DarwinCoreXmpMetadataModify(metaclass=CaseMeta):

    filename = path("$data_path/exiv2.dc.dwc.i18n_copy.jpg")
    dataname = path("$data_path/exiv2.dc.dwc.i18n.txt")

    commands = [
        "$exiv2 -pa $filename",
        "$exiv2 -m $dataname $filename",
        "$exiv2 -pa $filename"
    ]

    stdout = [
        "",
        "",
        """Xmp.dwc.Record                               XmpText     0  type="Struct"
Xmp.dwc.Record/dwc:basisOfRecord             XmpText    14  FossilSpecimen
Xmp.dwc.Record/dwc:collectionCode            XmpText     7  Mammals
Xmp.dwc.Record/dwc:collectionID              XmpText    29  urn:lsid:biocol.org:col:34818
Xmp.dwc.Record/dwc:dataGeneralizations       XmpText    23  Coordinates generalized
Xmp.dwc.Record/dwc:datasetID                 XmpText     3  MVZ
Xmp.dwc.Record/dwc:datasetName               XmpText    17  Grinnell Resurvey
Xmp.dwc.Record/dwc:informationWithheld       XmpText    20  location information
Xmp.dwc.Record/dwc:institutionCode           XmpText     3  CDS
Xmp.dwc.Record/dwc:institutionID             XmpText    14  Charles Darwin
Xmp.dwc.Record/dwc:ownerInstitutionCode      XmpText     3  NPS
Xmp.dwc.Record/dwc:dynamicProperties         XmpBag      1  targusLengthInMeters=0.014; weightInGrams=120
Xmp.dwc.Occurrence                           XmpText     0  type="Struct"
Xmp.dwc.Occurrence/dwc:behavior              XmpText     8  roosting
Xmp.dwc.Occurrence/dwc:catalogNumber         XmpText     9  2008.1334
Xmp.dwc.Occurrence/dwc:disposition           XmpText    13  in collection
Xmp.dwc.Occurrence/dwc:establishmentMeans    XmpText    10  cultivated
Xmp.dwc.Occurrence/dwc:individualCount       XmpText     2  23
Xmp.dwc.Occurrence/dwc:individualID          XmpText    10  U.amer. 44
Xmp.dwc.Occurrence/dwc:lifeStage             XmpText     3  egg
Xmp.dwc.Occurrence/dwc:occurrenceDetails     XmpText    50  http://mvzarctos.berkeley.edu/guid/MVZ:Mamm:165861
Xmp.dwc.Occurrence/dwc:occurrenceID          XmpText    28  urn:lsid:nhm.ku.edu:Herps:32
Xmp.dwc.Occurrence/dwc:occurrenceRemarks     XmpText    10  found dead
Xmp.dwc.Occurrence/dwc:occurrenceStatus      XmpText     7  present
Xmp.dwc.Occurrence/dwc:recordNumber          XmpText     8  OPP 7101
Xmp.dwc.Occurrence/dwc:reproductiveCondition XmpText    16  non-reproductive
Xmp.dwc.Occurrence/dwc:sex                   XmpText     6  female
Xmp.dwc.Occurrence/dwc:associatedMedia       XmpBag      1  http://arctos.database.museum/SpecimenImages/UAMObs/Mamm/2/P7291179.JPG
Xmp.dwc.Occurrence/dwc:associatedOccurrences XmpBag      1  sibling of
Xmp.dwc.Occurrence/dwc:associatedReferences  XmpBag      1  Christopher J.	89(3):755-76
Xmp.dwc.Occurrence/dwc:associatedSequences   XmpBag      1  GenBank:U34853.1
Xmp.dwc.Occurrence/dwc:associatedTaxa        XmpBag      1  host:Quercus alba
Xmp.dwc.Occurrence/dwc:otherCatalogNumbers   XmpBag      1  FNMH:Mammal:1234
Xmp.dwc.Occurrence/dwc:preparations          XmpBag      1  skin
Xmp.dwc.Occurrence/dwc:previousIdentifications XmpBag      1  Anthus sp.,
Xmp.dwc.Occurrence/dwc:recordedBy            XmpBag      1  Oliver P.
Xmp.dwc.Event                                XmpText     0  type="Struct"
Xmp.dwc.Event/dwc:day                        XmpText     2  31
Xmp.dwc.Event/dwc:earliestDate               XmpText    25  2012-09-03T00:00:00-06:00
Xmp.dwc.Event/dwc:latestDate                 XmpText    25  2013-01-27T00:00:00-06:00
Xmp.dwc.Event/dwc:endDayOfYear               XmpText     3  366
Xmp.dwc.Event/dwc:eventID                    XmpText     4  1234
Xmp.dwc.Event/dwc:eventTime                  XmpText    14  01:41:33+03:00
Xmp.dwc.Event/dwc:eventDate                  XmpText    25  1899-12-30T01:41:33+03:00
Xmp.dwc.Event/dwc:fieldNotes                 XmpText    15  notes available
Xmp.dwc.Event/dwc:fieldNumber                XmpText     6  RV Sol
Xmp.dwc.Event/dwc:habitat                    XmpText    11  oak savanna
Xmp.dwc.Event/dwc:month                      XmpText     2  10
Xmp.dwc.Event/dwc:samplingEffort             XmpText    14  40 trap-nights
Xmp.dwc.Event/dwc:samplingProtocol           XmpText    13  Penguins from
Xmp.dwc.Event/dwc:startDayOfYear             XmpText     1  1
Xmp.dwc.Event/dwc:verbatimEventDate          XmpText    11  spring 1910
Xmp.dwc.Event/dwc:year                       XmpText     4  1999
Xmp.dwc.Event/dwc:eventRemarks               XmpText    16  lang=en-us after
Xmp.dwc.dctermsLocation                      XmpText     0  type="Struct"
Xmp.dwc.dctermsLocation/dwc:continent        XmpText    10  Antarctica
Xmp.dwc.dctermsLocation/dwc:coordinatePrecision XmpText     7  0.00001
Xmp.dwc.dctermsLocation/dwc:coordinateUncertaintyInMeters XmpText     2  30
Xmp.dwc.dctermsLocation/dwc:country          XmpText     7  Denmark
Xmp.dwc.dctermsLocation/dwc:countryCode      XmpText     2  AR
Xmp.dwc.dctermsLocation/dwc:county           XmpText     8  Missoula
Xmp.dwc.dctermsLocation/dwc:decimalLatitude  XmpText    11  -41.0983423
Xmp.dwc.dctermsLocation/dwc:decimalLongitude XmpText    12  -121.1761111
Xmp.dwc.dctermsLocation/dwc:footprintSRS     XmpText    12  GCS_WGS_1984
Xmp.dwc.dctermsLocation/dwc:footprintSpatialFit XmpText     1  1
Xmp.dwc.dctermsLocation/dwc:footprintWKT     XmpText    12  POLYGON ((10
Xmp.dwc.dctermsLocation/dwc:geodeticDatum    XmpText     5  WGS84
Xmp.dwc.dctermsLocation/dwc:georeferenceProtocol XmpText    34  MaNIS/HerpNet/ORNIS Georeferencing
Xmp.dwc.dctermsLocation/dwc:georeferenceRemarks XmpText    16  assumed distance
Xmp.dwc.dctermsLocation/dwc:georeferenceVerificationStatus XmpText    21  requires verification
Xmp.dwc.dctermsLocation/dwc:higherGeographyID XmpText    11  TGN:1002002
Xmp.dwc.dctermsLocation/dwc:island           XmpText    13  Isla Victoria
Xmp.dwc.dctermsLocation/dwc:islandGroup      XmpText    21  Alexander Archipelago
Xmp.dwc.dctermsLocation/dwc:locality         XmpText     5  25 km
Xmp.dwc.dctermsLocation/dwc:locationAccordingTo XmpText    15  Getty Thesaurus
Xmp.dwc.dctermsLocation/dwc:locationID       XmpText     7  EBT3000
Xmp.dwc.dctermsLocation/dwc:locationRemarks  XmpText    11  under water
Xmp.dwc.dctermsLocation/dwc:maximumDepthInMeters XmpText     3  100
Xmp.dwc.dctermsLocation/dwc:maximumDistanceAboveSurfaceInMeters XmpText     9  1.5 meter
Xmp.dwc.dctermsLocation/dwc:maximumElevationInMeters XmpText     3  200
Xmp.dwc.dctermsLocation/dwc:minimumDepthInMeters XmpText     4  25.5
Xmp.dwc.dctermsLocation/dwc:minimumDistanceAboveSurfaceInMeters XmpText     9  1.5 meter
Xmp.dwc.dctermsLocation/dwc:minimumElevationInMeters XmpText     3  100
Xmp.dwc.dctermsLocation/dwc:municipality     XmpText    10  Holzminden
Xmp.dwc.dctermsLocation/dwc:pointRadiusSpatialFit XmpText     1  1
Xmp.dwc.dctermsLocation/dwc:stateProvince    XmpText     7  Montana
Xmp.dwc.dctermsLocation/dwc:verbatimCoordinateSystem XmpText     3  UTM
Xmp.dwc.dctermsLocation/dwc:verbatimCoordinates XmpText     4  41 5
Xmp.dwc.dctermsLocation/dwc:verbatimDepth    XmpText     9  100-200 m
Xmp.dwc.dctermsLocation/dwc:verbatimElevation XmpText     9  100-200 m
Xmp.dwc.dctermsLocation/dwc:verbatimLatitude XmpText     4  41 5
Xmp.dwc.dctermsLocation/dwc:verbatimLocality XmpText     5  25 km
Xmp.dwc.dctermsLocation/dwc:verbatimLongitude XmpText     5  121 5
Xmp.dwc.dctermsLocation/dwc:verbatimSRS      XmpText     5  WGS84
Xmp.dwc.dctermsLocation/dwc:waterBody        XmpText    12  Indian Ocean
Xmp.dwc.dctermsLocation/dwc:georeferenceSources XmpBag      1  USGS 1:24,000
Xmp.dwc.dctermsLocation/dwc:georeferencedBy  XmpBag      1  Kristina Yamamoto
Xmp.dwc.dctermsLocation/dwc:higherGeography  XmpBag      1  South America;
Xmp.dwc.GeologicalContext                    XmpText     0  type="Struct"
Xmp.dwc.GeologicalContext/dwc:bed            XmpText     8  Lava Dam
Xmp.dwc.GeologicalContext/dwc:earliestAgeOrLowestStage XmpText     8  Atlantic
Xmp.dwc.GeologicalContext/dwc:earliestEonOrLowestEonothem XmpText    11  Phanerozoic
Xmp.dwc.GeologicalContext/dwc:earliestEpochOrLowestSeries XmpText    11  Pleistocene
Xmp.dwc.GeologicalContext/dwc:earliestEraOrLowestErathem XmpText     8  Cenozoic
Xmp.dwc.GeologicalContext/dwc:earliestPeriodOrLowestSystem XmpText     7  Neogene
Xmp.dwc.GeologicalContext/dwc:formation      XmpText    10  Notch Peak
Xmp.dwc.GeologicalContext/dwc:geologicalContextID XmpText     4  1234
Xmp.dwc.GeologicalContext/dwc:group          XmpText    10  Notch Peak
Xmp.dwc.GeologicalContext/dwc:highestBiostratigraphicZone XmpText     8  Atlantic
Xmp.dwc.GeologicalContext/dwc:latestAgeOrHighestStage XmpText     6  Boreal
Xmp.dwc.GeologicalContext/dwc:latestEonOrHighestEonothem XmpText    11  Proterozoic
Xmp.dwc.GeologicalContext/dwc:latestEpochOrHighestSeries XmpText    14  Ibexian Series
Xmp.dwc.GeologicalContext/dwc:latestPeriodOrHighestSystem XmpText     8  Tertiary
Xmp.dwc.GeologicalContext/dwc:lithostratigraphicTerms XmpText    10  Supergroup
Xmp.dwc.GeologicalContext/dwc:lowestBiostratigraphicZone XmpText     6  Boreal
Xmp.dwc.GeologicalContext/dwc:member         XmpText     8  Lava Dam
Xmp.dwc.Identification                       XmpText     0  type="Struct"
Xmp.dwc.Identification/dwc:dateIdentified    XmpText    25  2011-01-27T00:17:00-18:00
Xmp.dwc.Identification/dwc:identificationID  XmpText     4  1234
Xmp.dwc.Identification/dwc:identificationQualifier XmpText    14  aff. agrifolia
Xmp.dwc.Identification/dwc:identificationRemarks XmpText    21  Distinguished between
Xmp.dwc.Identification/dwc:identificationReferences XmpBag      1  Aves del
Xmp.dwc.Identification/dwc:identifiedBy      XmpBag      1  James L.
Xmp.dwc.Identification/dwc:typeStatus        XmpBag      1  holotype of
Xmp.dwc.Taxon                                XmpText     0  type="Struct"
Xmp.dwc.Taxon/dwc:acceptedNameUsage          XmpText    14  Tamias minimus
Xmp.dwc.Taxon/dwc:acceptedNameUsageID        XmpText    36  8fa58e08-08de-4ac1-b69c-1235340b7001
Xmp.dwc.Taxon/dwc:class                      XmpText    10  Vertebrata
Xmp.dwc.Taxon/dwc:family                     XmpText     7  Felidae
Xmp.dwc.Taxon/dwc:genus                      XmpText     4  Puma
Xmp.dwc.Taxon/dwc:infraspecificEpithet       XmpText     8  concolor
Xmp.dwc.Taxon/dwc:kingdom                    XmpText     8  Animalia
Xmp.dwc.Taxon/dwc:nameAccordingTo            XmpText    21  McCranie, J. comments
Xmp.dwc.Taxon/dwc:nameAccordingToID          XmpText    33  doi:10.1016/S0269-915X(97)80026-2
Xmp.dwc.Taxon/dwc:namePublishedIn            XmpText    10  Pearson O.
Xmp.dwc.Taxon/dwc:namePublishedInID          XmpText    29  http://hdl.handle.net/10199/7
Xmp.dwc.Taxon/dwc:namePublishedInYear        XmpText     4  2059
Xmp.dwc.Taxon/dwc:nomenclaturalCode          XmpText     4  ICBN
Xmp.dwc.Taxon/dwc:nomenclaturalStatus        XmpText    11  nom. ambig.
Xmp.dwc.Taxon/dwc:order                      XmpText     8  Mammalia
Xmp.dwc.Taxon/dwc:originalNameUsage          XmpText    22  Gasterosteus saltatrix
Xmp.dwc.Taxon/dwc:parentNameUsage            XmpText     9  Rubiaceae
Xmp.dwc.Taxon/dwc:parentNameUsageID          XmpText    36  8fa58e08-08de-4ac1-b69c-1235340b7001
Xmp.dwc.Taxon/dwc:phylum                     XmpText     8  Chordata
Xmp.dwc.Taxon/dwc:scientificName             XmpText    19  Ctenomys sociabilis
Xmp.dwc.Taxon/dwc:scientificNameAuthorship   XmpText    12  (Torr.) J.T.
Xmp.dwc.Taxon/dwc:scientificNameID           XmpText    35  urn:lsid:ipni.org:names:37829-1:1.3
Xmp.dwc.Taxon/dwc:specificEpithet            XmpText     8  concolor
Xmp.dwc.Taxon/dwc:subgenus                   XmpText     4  Puma
Xmp.dwc.Taxon/dwc:taxonConceptID             XmpText    36  8fa58e08-08de-4ac1-b69c-1235340b7001
Xmp.dwc.Taxon/dwc:taxonID                    XmpText    36  8fa58e08-08de-4ac1-b69c-1235340b7001
Xmp.dwc.Taxon/dwc:taxonRank                  XmpText    10  subspecies
Xmp.dwc.Taxon/dwc:taxonRemarks               XmpText    24  lang=en-us this name ...
Xmp.dwc.Taxon/dwc:taxonomicStatus            XmpText     7  invalid
Xmp.dwc.Taxon/dwc:verbatimTaxonRank          XmpText    12  Agamospecies
Xmp.dwc.Taxon/dwc:higherClassification       XmpBag      1  Animalia;Chordata;Vertebrata;Mammalia;Theria;Eutheria
Xmp.dwc.Taxon/dwc:vernacularName             LangAlt     2  lang="es-ES" Puma, lang="en-US" Cougar
Xmp.dwc.MeasurementOrFact                    XmpText     0  type="Struct"
Xmp.dwc.MeasurementOrFact/dwc:measurementAccuracy XmpText     4  0.01
Xmp.dwc.MeasurementOrFact/dwc:measurementDeterminedDate XmpText    25  2013-01-27T00:00:00-06:00
Xmp.dwc.MeasurementOrFact/dwc:measurementID  XmpText     4  1234
Xmp.dwc.MeasurementOrFact/dwc:measurementMethod XmpText    20  barometric altimeter
Xmp.dwc.MeasurementOrFact/dwc:measurementRemarks XmpText     6  tip of
Xmp.dwc.MeasurementOrFact/dwc:measurementType XmpText    11  tail length
Xmp.dwc.MeasurementOrFact/dwc:measurementUnit XmpText     2  mm
Xmp.dwc.MeasurementOrFact/dwc:measurementValue XmpText     2  45
Xmp.dwc.MeasurementOrFact/dwc:measurementDeterminedBy XmpBag      1  Javier de
Xmp.dwc.ResourceRelationship                 XmpText     0  type="Struct"
Xmp.dwc.ResourceRelationship/dwc:relatedResourceID XmpText     4  1234
Xmp.dwc.ResourceRelationship/dwc:relationshipAccordingTo XmpText    14  Julie Woodruff
Xmp.dwc.ResourceRelationship/dwc:relationshipEstablishedDate XmpText    25  2013-01-27T00:00:00-06:00
Xmp.dwc.ResourceRelationship/dwc:relationshipOfResource XmpText    12  duplicate of
Xmp.dwc.ResourceRelationship/dwc:relationshipRemarks XmpText    19  pollinator captured
Xmp.dwc.ResourceRelationship/dwc:resourceID  XmpText     4  1234
Xmp.dwc.ResourceRelationship/dwc:resourceRelationshipID XmpText     4  1234
Xmp.dc.language                              XmpBag      1  latin
Xmp.dc.rights                                LangAlt     2  lang="es-ES" CC Alan Pater, lang="en-US" Alan Pater CC
"""
    ]
    stderr = [""] * 3
    retval = [0] * 3



# -*- coding: utf-8 -*-

import system_tests

class TestDefaultCommandGrep(metaclass=system_tests.CaseMeta):

    url = "https://github.com/Exiv2/exiv2/issues/818"

    filename = system_tests.path("$data_path/Reagan.jpg")
    commands = [
        "$exiv2     -g Date $filename",
        "$exiv2 -px -g Date $filename",
    ]
    retval = [0, 0]
    stderr =   [""] * 2
    retval =    [0] * 2

    stdout = [
        """Exif.Image.DateTime                          Ascii      20  2016:09:13 11:58:16
Exif.Photo.DateTimeOriginal                  Ascii      20  2004:06:21 23:37:53
Exif.Photo.DateTimeDigitized                 Ascii      20  2004:06:21 23:37:53
Iptc.Application2.DateCreated                Date        8  2004-06-21
Iptc.Application2.DigitizationDate           Date        8  2004-06-21
Xmp.xmp.ModifyDate                           XmpText    25  2016-09-13T11:58:16+01:00
Xmp.xmp.CreateDate                           XmpText    25  2004-06-21T23:37:53+01:00
Xmp.xmp.MetadataDate                         XmpText    25  2016-09-13T11:58:16+01:00
Xmp.photoshop.DateCreated                    XmpText    10  2004-06-21\n""",
        """Xmp.xmp.ModifyDate                           XmpText    25  2016-09-13T11:58:16+01:00
Xmp.xmp.CreateDate                           XmpText    25  2004-06-21T23:37:53+01:00
Xmp.xmp.MetadataDate                         XmpText    25  2016-09-13T11:58:16+01:00
Xmp.photoshop.DateCreated                    XmpText    10  2004-06-21\n"""
    ]



# -*- coding: utf-8 -*-

import configparser
import os
import inspect
import subprocess
import threading
import shlex
import sys
import shutil
import string
import unittest


from bash_tests import utils as BT


if sys.platform in [ 'win32', 'msys', 'cygwin' ]:
    #: invoke subprocess.Popen with shell=True on Windows
    _SUBPROCESS_SHELL = True

    def _cmd_splitter(cmd):
        return cmd

    def _process_output_post(output):
        return output.replace('\r\n', '\n')

else:
    #: invoke subprocess.Popen with shell=False on Unix
    _SUBPROCESS_SHELL = False

    def _cmd_splitter(cmd):
        return shlex.split(cmd)

    def _process_output_post(output):
        return output


def _disjoint_dict_merge(d1, d2):
    """
    Merges two dictionaries whose keys are disjoint sets and returns the
    resulting dictionary:

    >>> d1 = {"a": 1}
    >>> d2 = {"b": 2, "c": 3}
    >>> _disjoint_dict_merge(d1, d2) == {"a": 1, "b": 2, "c": 3}
    True

    Passing dictionaries that share keys raises a ValueError:
    >>> _disjoint_dict_merge({"a": 1, "b": 6}, {"b": 2, "a": 3})
    Traceback (most recent call last):
     ..
    ValueError: Dictionaries have common keys.

    """
    inter = set(d1.keys()).intersection(set(d2.keys()))
    if len(inter) > 0:
        raise ValueError("Dictionaries have common keys.")
    res = d1.copy()
    res.update(d2)
    return res


class CasePreservingConfigParser(configparser.ConfigParser):
    r""" ConfigParser where the keys are case sensitive.

    The default ConfigParser converts all options in the config file with their
    lowercase version. This class overrides the respective functions and
    preserves the case of keys.

    The default behavior of ConfigParser:
    >>> conf_string = "[Section1]\nKey = Value"
    >>> default_conf = configparser.ConfigParser()
    >>> default_Config.read_string(conf_string)
    >>> list(default_conf['Section1'].keys())
    ['key']

    This classes' behavior:
    >>> case_preserve = CasePreservingConfigParser()
    >>> case_preserve.read_string(conf_string)
    >>> list(case_preserve['Section1'].keys())
    ['Key']
    """

    def optionxform(self, option):
        return option


#: global parameters extracted from the test suite's configuration file
_parameters = {}

#: variables extracted from the test suite's configuration file
_config_variables = {}

#: setting whether debug mode is enabled or not
_debug_mode = False


def set_debug_mode(debug):
    """ Enable or disable debug mode

    In debug mode the test suite will print out all commands that it runs, the
    expected output and the actually obtained output
    """
    global _debug_mode
    _debug_mode = debug


def configure_suite(config_file):
    """
    Populates a global datastructure with the parameters from the suite's
    configuration file.

    This function performs the following steps:
    1. read in the file ``config_file`` via the ConfigParser module using
       extended interpolation
    2. check that the sections ``variables`` and ``paths`` are disjoint
    3. extract the environment variables given in the ``ENV`` section
    4. save all entries from the ``variables`` section in the global
       datastructure
    5. interpret all entries in the ``paths`` section as relative paths from
       the configuration file, expand them to absolute paths and save them in
       the global datastructure

    For further information concerning the rationale behind this, please
    consult the documentation in ``doc.md``.
    """

    if not os.path.exists(config_file):
        raise ValueError(
            "Test suite config file {:s} does not exist"
            .format(os.path.abspath(config_file))
        )

    config = CasePreservingConfigParser(
        interpolation=configparser.ExtendedInterpolation(),
        delimiters=(':'),
        comment_prefixes=('#')
    )
    config.read(config_file)

    _parameters["suite_root"] = os.path.split(os.path.abspath(config_file))[0]

    if 'variables' in config and 'paths' in config:
        intersecting_keys = set(config["paths"].keys()) \
            .intersection(set(config["variables"].keys()))
        if len(intersecting_keys) > 0:
            raise ValueError(
                "The sections 'paths' and 'variables' must not share keys, "
                "but they have the following common key{:s}: {:s}"
                .format(
                    's' if len(intersecting_keys) > 1 else '',
                    ', '.join(k for k in intersecting_keys)
                )
            )

    # Extract the environment variables according to config['ENV'].
    # When an environment variable does not exist, set its default value according to config['ENV fallback'].
    for key in config['ENV']:
        env_name            = config['ENV'][key]
        env_fallback        = config['ENV fallback'].get(key, '')
        config['ENV'][key]  = os.environ.get(env_name, env_fallback)

    if 'variables' in config:
        for key in config['variables']:
            _config_variables[key] = config['variables'][key]

    if 'paths' in config:
        for key in config['paths']:
            rel_path = config['paths'][key]
            abs_path = os.path.abspath(
                os.path.join(_parameters["suite_root"], rel_path)
            )
            if key == "tmp_path" and not os.path.isdir(abs_path):
                os.mkdir(abs_path)
            if key == "data_path" and not os.path.exists(abs_path):
                raise ValueError(
                    "Path replacement for {short}: {abspath} does not exist"
                    " (was expanded from {rel})".format(
                        short=key,
                        abspath=abs_path,
                        rel=rel_path)
                )
            _config_variables[key] = abs_path

    for key in _config_variables:
        if key in globals():
            raise ValueError("Variable name {!s} already used.")

        globals()[key] = _config_variables[key]

    _parameters["timeout"] = config.getfloat(
        "General", "timeout", fallback=1.0
    )

    if 'memcheck' in config['General']:
        if config['General']['memcheck'] != '':
            _parameters['memcheck'] = config['General']['memcheck']
            _parameters["timeout"] *= config.getfloat(
                "General", "memcheck_timeout_penalty", fallback=20.0
            )
    
    # Configure the parameters for bash tests
    BT.Config.bin_dir           = os.path.abspath(config['ENV']['exiv2_path'])
    BT.Config.dyld_library_path = os.path.abspath(config['ENV']['dyld_library_path'])
    BT.Config.ld_library_path   = os.path.abspath(config['ENV']['ld_library_path'])
    BT.Config.data_dir          = os.path.abspath(config['paths']['data_path'])
    BT.Config.tmp_dir           = os.path.abspath(config['paths']['tmp_path'])
    BT.Config.exiv2_http        = config['ENV']['exiv2_http']
    BT.Config.exiv2_port        = config['ENV']['exiv2_port']
    BT.Config.exiv2_echo        = config['ENV']['exiv2_echo']
    BT.Config.verbose           = config['ENV']['verbose']
    BT.Config.valgrind          = config['ENV']['valgrind']


class FileDecoratorBase(object):
    """
    Base class for decorators that manipulate files for test cases.

    The decorator expects to be provided with at least one file path
    on construction. When called, it replaces the setUp() and
    tearDown() functions of the type it is called on with custom ones.

    The new setUp() function performs the following steps:
    - create a file list in the decorated class with the name stored in
      FILE_LIST_NAME (defaults to _files)
    - iterate over all files, performing:
        - expand the file's path via expand_variables (member function
          of the decorated class)
        - call self.setUp_file_action(expanded file name)
        - append the result to the file list in the decorated class
    - call the old setUp()

    The function self.setUp_file_action is provided by this class and
    is intended to be overridden by child classes to provide some
    functionality, like file copies.


    The new tearDown() function performs the following steps:
    - call the old tearDown() function
    - iterate over all files in the file list:
         - call self.tearDown_file_action(filename)

    The function self.tearDown_file_action can be overridden by child
    classes. The default version provided by this class simply deletes
    all files that are passed to it.


    Example
    -------

    We'll inherit from FileDecoratorBase and override the member
    functions setUp_file_action and tearDown_file_action:

    >>> class TestDecorator(FileDecoratorBase):
    ...     def setUp_file_action(self, f):
    ...         print("setUp_file_action with", f)
    ...         return f.capitalize()
    ...
    ...     def tearDown_file_action(self, f):
    ...         print("tearDown_file_action with", f)

    Then, we use that decorator to wrap a class mocking
    system_tests.Case:

    >>> @TestDecorator("one", "two", "three")
    ... class MockCase(object):
    ...     def setUp(self):
    ...         print("calling MockCase.setUp()")
    ...
    ...     def tearDown(self):
    ...         print("calling MockCase.tearDown()")
    ...
    ...     def expand_variables(self, var):
    ...         return var + "_file"

    >>> M = MockCase()

    setUp has been replaced by a the new version, but the old one is
    still called. The new setUp iterates over all parameters passed to
    the constructor of the decorator, passes them to expand_variables
    and then to setUp_file_action:
    >>> M.setUp()
    setUp_file_action with one_file
    setUp_file_action with two_file
    setUp_file_action with three_file
    calling MockCase.setUp()

    The tearDown() function works accordingly:
    >>> M.tearDown()
    calling MockCase.tearDown()
    tearDown_file_action with One_file
    tearDown_file_action with Two_file
    tearDown_file_action with Three_file

    Please note the capitalized "file" names (this is due to
    setUp_file_action returning f.capitalized()) and that the old
    tearDown is called after the new one runs.
    """

    #: Name of the attribute in the decorated child class where the list of
    #: files is stored
    FILE_LIST_NAME = '_files'

    def __init__(self, *files):
        """
        Constructor of FileDecoratorBase.

        To prevent accidental wrong usage, it raises an exception if
        it is not called as a decorator with parameters.

        Only the following syntax works for this decorator:
        >>> @FileDecoratorBase("test")
        ... class Test(unittest.TestCase):
        ...     pass

        Calling it without parameters or without parenthesis raises an
        exception:
        >>> @FileDecoratorBase()
        ... class Test(unittest.TestCase):
        ...     pass
        Traceback (most recent call last):
         ..
        ValueError: No files supplied.

        >>> @FileDecoratorBase
        ... class Test(unittest.TestCase):
        ...     pass
        Traceback (most recent call last):
         ..
        UserWarning: Decorator used wrongly, must be called with filenames in parenthesis
        """
        if len(files) == 0:
            raise ValueError("No files supplied.")
        elif len(files) == 1:
            if isinstance(files[0], type):
                raise UserWarning(
                    "Decorator used wrongly, must be called with "
                    "filenames in parenthesis"
                )

        self.files = files

    def new_setUp(self, old_setUp):
        """
        Returns a new setUp() function that can be used as a class
        member function (i.e. invoked via self.setUp()).

        Its functionality is described in this classes' docstring.
        """

        def setUp(other):
            if hasattr(other, self.FILE_LIST_NAME):
                raise TypeError(
                    "{!s} already has an attribute with the name {!s} which "
                    "would be overwritten by setUp()"
                    .format(other, self.FILE_LIST_NAME)
                )
            setattr(other, self.FILE_LIST_NAME, [])
            for f in self.files:
                expanded_fname = other.expand_variables(f)
                getattr(other, self.FILE_LIST_NAME).append(
                    self.setUp_file_action(expanded_fname)
                )
            old_setUp(other)
        return setUp

    def setUp_file_action(self, expanded_file_name):
        """
        This function is called on each file that is passed to the
        constructor during the call of the decorated class' setUp().

        Parameters:
        - expanded_file_name: the file's path expanded via
                              expand_variables from system_tests.Case

        Returns:
        This function should return a path that will be stored in the decorated
        class' file list (the name is given by the attribute
        FILE_LIST_NAME). The custom tearDown() function (that is returned by
        self.new_tearDown()) iterates over this list and invokes
        self.tearDown_file_action on each element in that list.
        E.g. if a child class creates file copies, that should be deleted after
        the test ran, then one would have to return the path of the copy, so
        that tearDown() can delete the copies.

        The default implementation does nothing.
        """
        pass

    def new_tearDown(self, old_tearDown):
        """
        Returns a new tearDown() function that can be used as a class
        member function.

        It's functionality is described in this classes' docstring.
        """

        def tearDown(other):
            old_tearDown(other)
            for f in getattr(other, self.FILE_LIST_NAME):
                self.tearDown_file_action(f)

        return tearDown

    def tearDown_file_action(self, f):
        """
        This function is called on each file in the decorated class'
        file list (that list is populated during setUp()).

        It can be used to perform cleanup operations after a test run.

        Parameters:
        - f: An element of the file list

        Returns:
        The return value is ignored

        The default implementation removes f.
        """
        os.remove(f)

    def __call__(self, cls):
        """
        Call operator for the usage as a decorator. It is
        automatically used by Python when this class is used as a
        decorator.

        Parameters:
        - cls: The decorated type. Must be a type

        Returns:
        - cls where the setUp and tearDown functions have been
          replaced by the functions that are returned by
          self.new_setUp() and self.new_tearDown()
        """
        if not isinstance(cls, type):
            raise ValueError("The decorator must be called on a type")
        old_setUp = cls.setUp
        cls.setUp = self.new_setUp(old_setUp)

        old_tearDown = cls.tearDown
        cls.tearDown = self.new_tearDown(old_tearDown)

        return cls


class CopyFiles(FileDecoratorBase):
    """
    Decorator for subclasses of system_test.Case that automatically creates a
    copy of the files specified as the parameters passed to the decorator.

    Example:
    >>> @CopyFiles("$some_var/file.txt", "$another_var/other_file.png")
    ... class Foo(Case):
    ...     pass

    The decorator will inject a new setUp method that at first calls the
    already defined setUp(), then expands all supplied file names using
    Case.expand_variables and then creates copies by appending '_copy' before
    the file extension. The paths to the copies are stored in
    self._copied_files.

    The decorator also injects a new tearDown method that deletes all files in
    self._files and then calls the original tearDown method.

    This function will also complain if it is called without arguments or
    without parenthesis, which is valid decorator syntax but is obviously a bug
    in this case as it can result in tests not being run without a warning.
    """

    #: override the name of the file list
    FILE_LIST_NAME = '_copied_files'

    def setUp_file_action(self, expanded_file_name):
        fname, ext = os.path.splitext(expanded_file_name)
        new_name = fname + '_copy' + ext
        return shutil.copyfile(expanded_file_name, new_name)
   
class CopyTmpFiles(FileDecoratorBase):
    """
    This class copies files from test/data to test/tmp
    Copied files are NOT removed in tearDown
    Example: @CopyTmpFiles("$data_path/test_issue_1180.exv")
    """

    #: override the name of the file list
    FILE_LIST_NAME = '_tmp_files'

    def setUp_file_action(self, expanded_file_name):
        tmp_path   = _config_variables['tmp_path']
        tmp_name   = os.path.join(tmp_path,os.path.basename(expanded_file_name))
        return shutil.copyfile(expanded_file_name, tmp_name)

    def tearDown_file_action(self, f):
        """
        Do nothing.   We don't clean up TmpFiles
        """

class DeleteFiles(FileDecoratorBase):
    """
    Decorator for subclasses of system_test.Case that automatically deletes all
    files specified as the parameters passed to the decorator after the test
    were run.

    Example:
    >>> @DeleteFiles("$some_var/an_output_file", "auxiliary_output.bin")
    ... class Foo(Case):
    ...     pass

    The decorator injects new setUp() and tearDown() functions. The new setUp()
    at first calls the old setUp() and then saves all files that should be
    deleted later in self._files_to_delete. The new tearDown() actually deletes
    all files supplied to the decorator and then runs the original tearDown()
    function.
    """

    #: override the name of the file list
    FILE_LIST_NAME = '_files_to_delete'

    def setUp_file_action(self, expanded_file_name):
        return expanded_file_name


def path(path_string):
    r"""
    Converts a path which uses ``/`` as a separator into a path which uses the
    path separator of the current operating system.

    Example
    -------

    >>> import platform
    >>> sep = "\\" if platform.system() == "Windows" else "/"
    >>> path("a/b") == "a" + sep + "b"
    True
    >>> path("a/more/complex/path") == sep.join(['a', 'more', 'complex', 'path'])
    True
    """
    return os.path.join(*path_string.split('/'))


    """
    This function reads in the attributes commands, retval, stdout, stderr,
    stdin and runs the `expand_variables` function on each. The resulting
    commands are then run using the subprocess module and compared against the
    expected values that were provided in the attributes via `compare_stdout`
    and `compare_stderr`. Furthermore a threading.Timer is used to abort the
    execution if a configured timeout is reached.

    This function is automatically added as a member function to each system
    test by the CaseMeta metaclass. This ensures that it is run by each system
    test **after** setUp() and setUpClass() were run.
    """
def test_run(self):
    if not (len(self.commands) == len(self.retval)
            == len(self.stdout) == len(self.stderr) == len(self.stdin)):
        raise ValueError(
            "commands, retval, stdout, stderr and stdin don't have the same "
            "length"
        )

    for i, command, retval, stdout, stderr, stdin in \
        zip(range(len(self.commands)),
            self.commands,
            self.retval,
            self.stdout,
            self.stderr,
            self.stdin):
        command, retval, stdout, stderr, stdin = [
            self.expand_variables(var) for var in
            (command, retval, stdout, stderr, stdin)
        ]

        retval = int(retval)

        if "memcheck" in _parameters:
            command = _parameters["memcheck"] + " " + command

        if _debug_mode:
            print(
                '', "="*80, "will run: " + command, "expected stdout:", stdout,
                "expected stderr:", stderr,
                "expected return value: {:d}".format(retval),
                sep='\n'
            )

        proc = subprocess.Popen(
            _cmd_splitter(command),
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            stdin=subprocess.PIPE if stdin is not None else None,
            env=self._get_env(),
            cwd=self.work_dir,
            shell=_SUBPROCESS_SHELL
        )

        # Setup a threading.Timer which will terminate the command if it takes
        # too long. Don't use the timeout parameter in subprocess.Popen, since
        # that is not available for all Python 3 versions.
        # Use a dictionary to indicate a timeout, as booleans get passed by
        # value and the changes made timeout_reached function will not be
        # visible once it exits (the command will still be terminated once the
        # timeout expires).
        timeout = {"flag": False}

        def timeout_reached(tmout):
            tmout["flag"] = True
            proc.kill()

        t = threading.Timer(
            _parameters["timeout"], timeout_reached, args=[timeout]
        )

        def get_encode_err():
            """ Return an error message indicating that the encoding of stdin
            failed.
            """
            return "Could not encode stdin {!s} for the command {!s} with the"\
                " following encodings: {!s}"\
                .format(stdin, command, ','.join(self.encodings))

        # Prepare stdin: try to encode it or keep it at None if it was not
        # provided
        encoded_stdin = None
        if stdin is not None:
            encoded_stdin = self._encode(
                stdin, lambda data_in, encoding: data_in.encode(encoding),
                get_encode_err
            )

        if _debug_mode:
            print('', "stdin:", stdin or "", sep='\n')

        t.start()
        got_stdout, got_stderr = proc.communicate(input=encoded_stdin)
        t.cancel()

        def get_decode_error():
            """ Return an error indicating the the decoding of stdout/stderr
            failed.
            """
            return "Could not decode the output of the command '{!s}' with "\
                "the following encodings: {!s}"\
                .format(command, ','.join(self.encodings))

        def decode_output(data_in, encoding):
            """ Decode stdout/stderr, consider platform dependent line
            endings.
            """
            return _process_output_post(data_in.decode(encoding))

        processed_stdout, processed_stderr = [
            self._encode(output, decode_output, get_decode_error)
            for output in (got_stdout, got_stderr)
        ]

        if _debug_mode:
            print(
                "got stdout:", processed_stdout, "got stderr:",
                processed_stderr, "got return value: {:d}"
                .format(proc.returncode),
                sep='\n'
            )

        self.assertFalse(timeout["flag"], msg="Timeout reached")
        self.compare_stderr(i, command, processed_stderr, stderr)
        self.compare_stdout(i, command, processed_stdout, stdout)
        self.assertEqual(
            retval, proc.returncode, msg="Return value does not match"
        )

        self.post_command_hook(i, command)

    self.post_tests_hook()


class Case(unittest.TestCase):
    """
    System test case base class, provides the functionality to interpret static
    class members as system tests.

    The class itself only provides utility functions and system tests need not
    inherit from it, as it is automatically added via the CaseMeta metaclass.
    """

    #: maxDiff set so that arbitrarily large diffs will be shown
    maxDiff = None

    #: list of encodings that are used to decode the test program's output
    #: the first encoding that does not raise a UnicodeError is used
    encodings = ['utf-8', 'iso-8859-1']

    inherit_env = True

    @classmethod
    def setUpClass(cls):
        """
        This function adds the variable work_dir to the class, which is the
        path to the directory where the python source file is located.
        """
        cls.work_dir = os.path.dirname(inspect.getfile(cls))

    def _get_env(self):
        """ Return an appropriate env value for subprocess.Popen.

        This function returns either an appropriately populated dictionary or
        None (the latter if this class has no attribute env). If a dictionary
        is returned, then it will be either exactly self.env (when inherit_env
        is False) or a copy of the current environment merged with self.env
        (the values from self.env take precedence).
        """
        if not hasattr(self, "env"):
            return None

        if not self.inherit_env:
            return self.env

        env_copy = os.environ.copy()
        for key in self.env:
            env_copy[key] = self.env[key]

        return env_copy

    def _encode(self, data_in, encode_action, get_err):
        """
        Try to convert data_in via encode_action using the encodings in
        self.encodings.

        This function tries all encodings in self.encodings to run
        encode_action with the parameters (data_in, encoding). If encode_action
        raises a UnicodeError, the next encoding is used, otherwise the result
        of encode_action is returned. If an encoding is equal to the type
        bytes, then data_in is returned unmodified.

        If all encodings result in a UnicodeError, then the conversion is
        considered unsuccessful and get_err() is called to obtain an error
        string which is raised as a ValueError.
        """
        result = None
        for encoding in self.encodings:
            if encoding == bytes:
                return data_in
            try:
                result = encode_action(data_in, encoding)
            except UnicodeError:
                pass
            else:
                break
        if result is None:
            raise ValueError(get_err())

        return result

    def _compare_output(self, i, command, got, expected, msg=None):
        """ Compares the expected and actual output of a test case. """
        if isinstance(got, bytes):
            self.assertEqual(got, expected, msg=msg)
        else:
            self.assertMultiLineEqual(
                expected, got, msg=msg
            )

    def compare_stdout(self, i, command, got_stdout, expected_stdout):
        """
        Function to compare whether the expected & obtained stdout match.

        This function is automatically invoked by test_run with the following
        parameters:
        i - the index of the current command that is run in self.commands
        command - the command that was run
        got_stdout - the obtained stdout, post-processed depending on the
                     platform so that lines always end with \n
        expected_stdout - the expected stdout extracted from self.stdout

        The default implementation uses assertMultiLineEqual from
        unittest.TestCase for ordinary strings and assertEqual for binary
        output. This function can be overridden in a child class to implement a
        custom check.
        """
        self._compare_output(
            i, command, expected_stdout, got_stdout,
            msg="Standard output does not match"
        )

    def compare_stderr(self, i, command, got_stderr, expected_stderr):
        """ Same as compare_stdout only for standard-error. """
        self._compare_output(
            i, command, expected_stderr, got_stderr,
            msg="Standard error does not match"
        )

    def expand_variables(self, unexpanded_string):
        """
        Expands all variables of the form ``$var`` in the given string using
        the dictionary `variable_dict`.

        The expansion itself is performed by the string's template module using
        the function `safe_substitute`.

        If unexpanded_string is of the type bytes, then no expansion is
        performed.
        """
        if isinstance(unexpanded_string, bytes) or unexpanded_string is None:
            return unexpanded_string

        return string.Template(str(unexpanded_string))\
            .safe_substitute(**self.variable_dict)

    def post_command_hook(self, i, command):
        """ Function that is run after the successful execution of one command.

        It is invoked with the following parameters:
        i - the index of the current command that is run in self.commands
        command - the command that was run

        It should return nothing.

        This function can be overridden to perform additional checks after the
        command ran, for instance it can check whether files were created.

        The default implementation does nothing.
        """
        pass

    def post_tests_hook(self):
        """
        Function that is run after the successful execution all commands. It
        should return nothing.

        This function can be overridden to run additional checks that only make
        sense after all commands ran.

        The default implementation does nothing.
        """
        pass


class CaseMeta(type):
    """ System tests generation metaclass.

    This metaclass is performs the following tasks:

    1. Add the `test_run` function as a member of the test class
    2. Add the `Case` class as the parent class
    3. Expand all variables already defined in the class, so that any
       additional code does not have to perform this task

    Using a metaclass instead of inheriting from Case has the advantage, that
    we can expand all variables in the strings before any test case or even the
    class constructor is run! Thus users will immediately see the expanded
    result. Also adding the `test_run` function as a direct member and not via
    inheritance enforces that it is being run **after** the test cases setUp &
    setUpClass (which oddly enough seems not to be the case in the unittest
    module where test functions of the parent class run before setUpClass of
    the child class).
    """

    def __new__(mcs, clsname, bases, dct):

        assert len(_parameters) != 0, \
            "Internal error: substitution dictionary not populated"

        changed = True

        # expand all non-private variables by brute force
        # => try all expanding all elements defined in the current class until
        # there is no change in them any more
        keys = [key for key in list(dct.keys()) if not key.startswith('_')]
        while changed:
            changed = False

            for key in keys:

                old_value = dct[key]

                # only try expanding strings and lists
                if isinstance(old_value, str):
                    new_value = string.Template(old_value).safe_substitute(
                        **_disjoint_dict_merge(dct, _config_variables)
                    )
                elif isinstance(old_value, list):
                    # do not try to expand anything but strings in the list
                    new_value = [
                        string.Template(elem).safe_substitute(
                            **_disjoint_dict_merge(dct, _config_variables)
                        )
                        if isinstance(elem, str) else elem
                        for elem in old_value
                    ]
                else:
                    continue

                if old_value != new_value:
                    changed = True
                    dct[key] = new_value

        dct['variable_dict'] = _disjoint_dict_merge(dct, _config_variables)
        dct['test_run'] = test_run

        if Case not in bases:
            bases += (Case,)

        CaseMeta.add_default_values(clsname, dct)

        return super(CaseMeta, mcs).__new__(mcs, clsname, bases, dct)

    @staticmethod
    def add_default_values(clsname, dct):
        if 'commands' not in dct:
            raise ValueError(
                "No member 'commands' in class {!s}.".format(clsname)
            )

        cmd_length = len(dct['commands'])

        for member, default in zip(
                ('stderr', 'stdout', 'stdin', 'retval'),
                ('', '', None, 0)):
            if member not in dct:
                dct[member] = [default] * cmd_length


def check_no_ASAN_UBSAN_errors(self, i, command, got_stderr, expected_stderr):
    """
    Drop-in replacement for the default Case.compare_stderr() function that
    **only** checks for any signs of ASAN (address sanitizer) and UBSAN
    (undefined behavior sanitizer).

    Parameters:
    - i, command, expected_stderr: ignored
    - got_stderr: checked for signs of ASAN und UBSAN error messages

    This function ignores the expected output to stderr! It is intended for
    test cases where standard error is filled with useless debugging
    messages/warnings that are not really relevant for the test and not worth
    storing in the test suite. This function can be used to still be able to
    catch ASAN & UBSAN error messages.

    Example usage
    -------------

    Override the default compare_stderr function in your subclass of Case with
    this function:
    >>> class TestNoAsan(Case):
    ...     compare_stderr = check_no_ASAN_UBSAN_errors

    >>> T = TestNoAsan()

    The new compare_stderr will only complain if there are strings inside the
    obtained stderr which could be an error reported by ASAN/UBSAN:
    >>> T.compare_stderr(0, "", "runtime error: load of value 190", "some output")
    Traceback (most recent call last):
     ..
    AssertionError: 'runtime error' unexpectedly found in 'runtime error: load of value 190'

    >>> T.compare_stderr(0, "", "SUMMARY: AddressSanitizer: heap-buffer-overflow", "")
    Traceback (most recent call last):
     ..
    AssertionError: 'AddressSanitizer' unexpectedly found in 'SUMMARY: AddressSanitizer: heap-buffer-overflow'

    It will not complain in all other cases, especially when expected_stderr
    and got_stderr do not match:
    >>> T.compare_stderr(0, "", "some output", "other output")

    This function also supports binary output:
    >>> ASAN_ERROR = bytes("SUMMARY: AddressSanitizer: heap-buffer-overflow", encoding='ascii')
    >>> T.compare_stderr(0, "", ASAN_ERROR, "other output")
    Traceback (most recent call last):
     ..
    AssertionError: b'AddressSanitizer' unexpectedly found in b'SUMMARY: AddressSanitizer: heap-buffer-overflow'
    """
    UBSAN_MSG = "runtime error"
    ASAN_MSG = "AddressSanitizer"

    if isinstance(got_stderr, bytes):
        self.assertNotIn(UBSAN_MSG.encode('ascii'), got_stderr)
        self.assertNotIn(ASAN_MSG.encode('ascii'), got_stderr)
        return

    self.assertNotIn(UBSAN_MSG, got_stderr)
    self.assertNotIn(ASAN_MSG, got_stderr)




# -*- coding: utf-8 -*-

import system_tests


class Sigma24_105mmRecognization(metaclass=system_tests.CaseMeta):

    url = "https://github.com/Exiv2/exiv2/issues/45"

    filename = "$data_path/exiv2-g45.exv"
    commands = ["$exiv2 -pa --grep lens/i " + filename]
    stdout = ["""Exif.CanonCs.LensType                        Short       1  Sigma 24-105mm F4 DG OS HSM | A
Exif.CanonCs.Lens                            Short       3  24.0 - 105.0 mm
Exif.CanonCf.LensAFStopButton                Short       1  0
Exif.Canon.LensModel                         Ascii      74  24-105mm F4 DG OS HSM | Art 013
"""]
    stderr = [""]
    retval = [0]



# -*- coding: utf-8 -*-

import system_tests


class TestCvePoC(metaclass=system_tests.CaseMeta):

    url = "https://github.com/Exiv2/exiv2/issues/138"

    filename = "$data_path/007-heap-buffer-over"
    commands = ["$exiv2 $filename"]
    stdout = [
        """File name       : $filename
File size       : 331696 Bytes
MIME type       : image/tiff
Image size      : 0 x 12336
Thumbnail       : None
Camera make     : 0000
Camera model    : 0000000000000
Image timestamp : 
File number     : 
Exposure time   : 
Aperture        : 
Exposure bias   : 
Flash           : 
Flash bias      : 
Focal length    : 
Subject distance: 
ISO speed       : 
Exposure mode   : 
Metering mode   : 
Macro mode      : 
Image quality   : 
White balance   : 
Copyright       : 
Exif comment    : 

"""
    ]
    stderr = [
        """Error: Offset of directory Image, entry 0x0100 is out of bounds: Offset = 0x30303030; truncating the entry
Warning: Directory Image, entry 0x0111: Strip 17 is outside of the data area; ignored.
Error: Directory Photo with 8224 entries considered invalid; not read.
Warning: Removing 913 characters from the beginning of the XMP packet
Error: XMP Toolkit error 201: Error in XMLValidator
Warning: Failed to decode XMP metadata.
"""
    ]
    retval = [0]



# -*- coding: utf-8 -*-

import system_tests

def stderr_exception (fname):
    return """$exiv2_exception_message """ + fname + """:
$kerFailedToReadImageData
"""

class PngReadRawProfile(metaclass=system_tests.CaseMeta):

    url = "https://github.com/Exiv2/exiv2/issues/428"

    filenames = [
        system_tests.path("$data_path/issue_428_poc1.png"),
        system_tests.path("$data_path/issue_428_poc3.png"),
        system_tests.path("$data_path/issue_428_poc4.png"),
        system_tests.path("$data_path/issue_428_poc5.png"),
        system_tests.path("$data_path/issue_428_poc8.png"),

        system_tests.path("$data_path/issue_428_poc2.png"),
        system_tests.path("$data_path/issue_428_poc6.png"),
        system_tests.path("$data_path/issue_428_poc7.png"),
    ]

    commands = ["$exiv2 " + fname for fname in filenames]
    stdout = [""] * len(filenames)
    stderr = [ stderr_exception(fname) for fname in filenames[0:5] ]
    stderr.append("""$exiv2_exception_message """ + filenames[5] + """:
$kerInputDataReadFailed
""")
    stderr.append("""Error: XMP Toolkit error 201: Error in XMLValidator
Warning: Failed to decode XMP metadata.
""" + stderr_exception(filenames[6]))
    stderr.append("""Warning: Failed to decode Exif metadata.
""" + stderr_exception(filenames[7]))

    retval = [1] * len(filenames)



# -*- coding: utf-8 -*-

import system_tests


class CorruptedIopDirectory(metaclass=system_tests.CaseMeta):

    url = "http://dev.exiv2.org/issues/495"

    filename = "$data_path/exiv2-bug495.jpg"

    commands = ["$exiv2 -u -pi $filename"]

    stdout = ["""Iptc.Application2.Caption                    String     56  Die Insel Stromboli mit dem Vulkan Stromboli, 1000m hoch
Iptc.Application2.Program                    String      7  digiKam
Iptc.Application2.ProgramVersion             String     11  0.9.0-beta3
"""]
    stderr = [""]
    retval = [0]

    compare_stderr = system_tests.check_no_ASAN_UBSAN_errors



# -*- coding: utf-8 -*-

import system_tests

class NikonTokinaLens_atx_i_11_16_F28(metaclass=system_tests.CaseMeta):
    url = "https://github.com/Exiv2/exiv2/issues/1078"

    filename = "$data_path/Tokina_atx-i_11-16mm_F2.8_CF.exv"
    commands = ["$exiv2 -pa --grep lensid/i $filename"]
    stderr = [""]
    stdout = [""
        """Exif.NikonLd3.LensIDNumber                   Byte        1  Tokina atx-i 11-16mm F2.8 CF
"""
]
    retval = [0]



import difflib
import fnmatch
import hashlib
import multiprocessing
import os
import platform
import re
import shlex
import shutil
import subprocess
import time
import sys
from   http   import server
from   urllib import request
import system_tests


"""
Part 1:
Here is the configuration part of test cases.
"""

class Config:
    # The configuration parameters for bash test
    # When you run the test cases through `python3 runner.py`, the function configure_suite() in system_tests.py will override these parameters.
    exiv2_dir           = os.path.normpath(os.path.join(os.path.abspath(__file__), '../../../'))
    bin_dir             = os.path.join(exiv2_dir, 'build/bin')
    if 'EXIV2_BINDIR' in os.environ:
        bin_dir         = os.environ['EXIV2_BINDIR']
    dyld_library_path   = os.path.join(bin_dir, '../lib')
    ld_library_path     = os.path.join(bin_dir, '../lib')
    data_dir            = os.path.join(exiv2_dir, 'test/data')
    tmp_dir             = os.path.join(exiv2_dir, 'test/tmp')
    system_name         = platform.system() or 'Unknown'    # It could be Windows, Linux, etc.
    exiv2_http          = 'http://127.0.0.1'
    exiv2_port          = '12760'
    valgrind            = ''
    if 'EXIV2_PORT' in os.environ:
        exiv2_port      = os.environ['EXIV2_PORT']
    if 'EXIV2_HTTP' in os.environ:
        exiv2_http      = os.environ['EXIV2_HTTP']
    if 'VALGRIND' in os.environ:
        valgrind        = os.environ['VALGRIND']

    @classmethod
    def init(cls):
        """ Init test environments and variables that may be modified """
        os.makedirs(cls.tmp_dir, exist_ok=True)
        os.chdir(cls.tmp_dir)
        log.buffer      = []
        cls.bin_files   = [i.split('.')[0] for i in os.listdir(cls.bin_dir)]
        cls.encoding    = 'utf-8'


"""
Part 2:
Here are some common functions that are poorly coupled with test cases.
"""


def find(directory='.', pattern=None, re_pattern=None, depth=-1, onerror=print) -> list:
    """
    Find files and directories that match the pattern in the specified directory and return their paths.
    Work in recursive mode. If there are thousands of files, the runtime may be several seconds.
    - `directory`   : Find files in this directory and its subdirectories
    - `pattern`     : Filter filename based on shell-style wildcards.
    - `re_pattern`  : Filter filename based on regular expressions.
    - `depth`       : Depth of subdirectories. If its value is negative, the depth is infinite.
    - `onerror`     : A callable parameter. it will be called if an exception occurs.

    Sample:
    >>> find(pattern='*.py')
    >>> find(re_pattern='.*.py')
    """
    if not os.path.isdir(directory):
        raise ValueError("{} is not an existing directory.".format(directory))

    try:
        file_list = os.listdir(directory)
    except PermissionError as e:    # Sometimes it does not have access to the directory
        onerror("PermissionError: {}".format(e))
        return []

    def match(name, pattern=None, re_pattern=None):
        if pattern and not fnmatch.fnmatch(name, pattern):
            return False
        if re_pattern and not re.findall(re_pattern, name):
            return False
        return True

    path_list = []
    if match(os.path.basename(directory), pattern, re_pattern):
        path_list.append(directory)
    if depth != 0:
        for filename in file_list:
            path = os.path.join(directory, filename)
            if os.path.isdir(path):
                path_list.extend(find(path, pattern, re_pattern, depth-1, onerror))
                continue
            if match(filename, pattern, re_pattern):
                path_list.append(path)

    return path_list


def cp(src, dst):
    """ Copy one or more files or directories. It simulates `cp -rf src dst`. """
    if os.path.isfile(src):
        shutil.copy(src, dst)
    elif os.path.isdir(src):
        if os.path.isdir(dst):
            dst_dir = os.path.join(dst, os.path.basename(src))
        else:
            dst_dir = dst
        for src_path in find(src):
            relpath = os.path.relpath(src_path, src)
            dst_path = os.path.join(dst_dir, relpath)
            if os.path.isdir(src_path):
                os.makedirs(dst_path, exist_ok=True)
            else:
                shutil.copy(src_path, dst_path)
    else:
        raise ValueError('src is not a valid path to a file or directory.')


def rm(*paths):
    """ Remove one or more files or directories. It simulates `rm -rf paths`. """
    for path in paths:
        if os.path.isfile(path):
            os.remove(path)
        elif os.path.isdir(path):
            for sub_path in find(path, depth=1)[1:]:
                if os.path.isdir(sub_path):
                    rm(sub_path)
                else:
                    os.remove(sub_path)
            os.rmdir(path)  # Remove the directory only when it is empty
        else:
            continue


def mv(src, dst):
    """ Move one or more files or directories. """
    cp(src, dst)
    rm(src)


def cat(*files, encoding=None, return_bytes=False):
    if return_bytes:
        result = b''
        for i in files:
            with open(i, 'rb') as f:
                result += f.read()
    else:
        result = ''
        for i in files:
            with open(i, 'r', encoding=encoding or Config.encoding) as f:
                result += f.read()
    return result


def grep(pattern, *files, encoding=None):
    result  = ''
    pattern = '.*{}.*'.format(pattern)
    for i in files:
        content = cat(i, encoding=encoding or Config.encoding)
        result += '\n'.join(re.findall(pattern, content))
    return result


def save(content: (bytes, str, tuple, list), filename, encoding=None):
    if isinstance(content, bytes):
        with open(filename, 'wb') as f:
            f.write(content)
        return
    if isinstance(content, (tuple, list)):
        content = '\n'.join(content)
    if isinstance(content, str):
        with open(filename, 'w', encoding=encoding or Config.encoding) as f:
            f.write(content)
    else:
        raise ValueError('Expect content of type (bytes, str, tuple, list), but get {}'.format(type(content).__name__))


def diff(file1, file2, encoding=None):
    """
    Simulates the output of GNU diff.
    You can use `diff(f1, f2)` to simulate `diff -w f1 f2`
    """
    encoding     = encoding or Config.encoding
    texts        = []
    for f in [file1, file2]:
        with open(f, encoding=encoding) as f:
            text = f.read()
        text     = text.replace('\r\n', '\n') # Ignore line breaks for Windows
        texts   += [text.split('\n')]
    text1, text2 = texts

    output       = []
    new_part     = True
    num          = 0
    for line in difflib.unified_diff(text1, text2, fromfile=file1, tofile=file2, n=0, lineterm=''):
        num     += 1
        if num   < 3:
            # line         = line.replace('--- ', '<<< ')
            # line         = line.replace('+++ ', '>>> ')
            # output      += [line]
            continue

        flag             = line[0]
        if flag         == '-':   # line unique to sequence 1
            new_flag     = '< '
        elif flag       == '+':   # line unique to sequence 2
            new_flag     = '> '
            if new_part:
                new_part = False
                output  += ['---']
        elif flag       == ' ':   # line common to both sequences
            # new_flag   = '  '
            continue
        elif flag       == '?':   # line not present in either input sequence
            new_flag     = '? '
        elif flag       == '@':
            output      += [re.sub(r'@@ -([^ ]+) \+([^ ]+) @@', r'\1c\2', line)]
            new_part     = True
            continue
        else:
            new_flag     = flag
        output          += [new_flag + line[1:]]

    return '\n'.join(output)


def diff_bytes(file1, file2, return_str=False):
    """
    Compare the bytes of two files.
    Simulates the output of GNU diff.
    """
    texts        = []
    for f in [file1, file2]:
        with open(f, 'rb') as f:
            text = f.read()
        text     = text.replace(b'\r\n', b'\n') # Ignore line breaks for Windows
        texts   += [text.split(b'\n')]
    text1, text2 = texts

    output       = []
    new_part     = True
    num          = 0
    for line in difflib.diff_bytes(difflib.unified_diff, text1, text2,
                                   fromfile=file1.encode(), tofile=file2.encode(), n=0, lineterm=b''):
        num     += 1
        if num   < 3:
            line         = line.decode()
            line         = line.replace('--- ', '<<< ')
            line         = line.replace('+++ ', '>>> ')
            output      += [line.encode()]
            continue

        flag             = line[0:1]
        if flag         == b'-':   # line unique to sequence 1
            new_flag     = b'< '
        elif flag       == b'+':   # line unique to sequence 2
            new_flag     = b'> '
            if new_part:
                new_part = False
                output  += [b'---']
        elif flag       == b' ':   # line common to both sequences
            # new_flag   = b'  '
            continue
        elif flag       == b'?':   # line not present in either input sequence
            new_flag     = b'? '
        elif flag       == b'@':
            output      += [re.sub(rb'@@ -([^ ]+) \+([^ ]+) @@', rb'\1c\2', line)]
            new_part     = True
            continue
        else:
            new_flag     = flag
        output          += [new_flag + line[1:]]

    if return_str:
        return '\n'.join([repr(line)[2:-1] for line in output])
    else:
        return b'\n'.join(output)


def md5sum(filename):
    """ Calculate the MD5 value of the file """
    with open(filename, "rb") as f:
        return hashlib.md5(f.read()).hexdigest()


def pretty_xml(text, encoding=None):
    """
    Add indent to the XML text
    """
    from lxml import etree
    encoding     = encoding or Config.encoding
    root = etree.fromstring(text)
    etree.indent(root)
    return etree.tostring(root).decode(encoding)


class Log:

    def __init__(self):
        self.buffer = []

    def to_str(self):
        return '\n'.join(self.buffer)

    def add(self, msg):
        self.buffer.append(str(msg))

    def info(self, msg, index=None):
        self.add('[INFO] {}'.format(msg))

    def warn(self, msg):
        self.add('[WARN] {}'.format(msg))

    def error(self, msg):
        self.add('[ERROR] {}'.format(msg))


log = Log()


class HttpServer:
    def __init__(self, bind='127.0.0.1', port=80, work_dir='.'):
        self.bind = bind
        self.port = int(port)
        self.work_dir = work_dir

    def _start(self):
        """ Equivalent to executing `python3 -m http.server` """
        os.chdir(self.work_dir)
        server.test(HandlerClass=server.SimpleHTTPRequestHandler, bind=self.bind, port=self.port)
        log.error('The HTTP server exits without calling stop()')
        print(log.to_str())

    def start(self):
        log.info('Starting HTTP server ...')
        self.proc = multiprocessing.Process(target=self._start, name=str(self))
        self.proc.start()
        time.sleep(2)
        try:
            with request.urlopen('http://127.0.0.1:{}'.format(self.port), timeout=3) as f:
                if f.status != 200:
                    raise RuntimeError()
        except:
            raise RuntimeError('Failed to run the HTTP server')
        log.info('The HTTP server started')

    def stop(self):
        self.proc.terminate()


"""
Part 3:
Here are some functions that are highly coupled to test cases.
"""

def copyTestFile(src, dst=''):
    """ Copy one test file from data_dir to tmp_dir """
    if not dst:
        dst = src
    shutil.copy(os.path.join(Config.data_dir, src),
                os.path.join(Config.tmp_dir, dst))


def diffCheck(file1, file2, in_bytes=False, encoding=None):
    """ Compare two files to see if they are different """
    if in_bytes:
        d = diff_bytes(file1, file2, return_str=True)
        if d:
            log.info('diff_bytes:\n' + d)
    else:
        d = diff(file1, file2, encoding=encoding or Config.encoding)
        if d:
            log.info('diff:\n' + d)
    return d == ''


def simply_diff(file1, file2, encoding=None):
    """ Find the first different line of the two text files """
    encoding    = encoding or Config.encoding
    list1       = cat(file1, encoding=encoding).split('\n')
    list2       = cat(file2, encoding=encoding).split('\n')
    if list1   == list2:
        return

    report      = []
    report     += ['{}: {} lines'.format(file1, len(list1))]
    report     += ['{}: {} lines'.format(file2, len(list2))]

    # Make them have the same number of lines
    max_lines   = max(len(list1), len(list2))
    for _list in [list1, list2]:
        _list  += [''] * (max_lines - len(_list))

    # Compare each line
    for i in range(max_lines):
        if list1[i] != list2[i]:
            report  += ['The first mismatch is in line {}:'.format(i + 1)]
            report  += ['< {}'.format(list1[i])]
            report  += ['> {}'.format(list2[i])]
            break
    return '\n'.join(report)


class Executer:
    """
    Execute a command in the shell, return a `Executer` object.
    - If a binary of Exiv2 is executed, the absolute path is automatically added.
    - `compatible_output=True`: filter out path delimiters, whitespace characters in output
    - `decode_output=True`: decode output from bytes to str

    Sample:
    >>> Executer('echo Hello').stdout
    >>> Executer('exiv2 --help').stdout
    """

    def __init__(self, cmd: str,
                 vars_dict=dict(),
                 cwd=None,
                 extra_env=dict(),
                 encoding=None,
                 stdin: (str, bytes) = None,
                 redirect_stderr_to_stdout=True,
                 assert_returncode=[0],
                 compatible_output=True,
                 decode_output=True):
        self.cmd            = cmd.format(**vars_dict)
        self.cwd            = cwd or Config.tmp_dir

        # set environment variables
        self.env            = os.environ.copy()
        self.env.update({'DYLD_LIBRARY_PATH': Config.dyld_library_path})
        self.env.update({'LD_LIBRARY_PATH': Config.ld_library_path})
        self.env.update({'TZ': 'GMT-8'})
        self.env.update(extra_env)

        self.encoding       = encoding or Config.encoding
        self.stdin          = stdin
        # self.stdout       = None
        # self.stderr       = None
        self.redirect_stderr_to_stdout  = redirect_stderr_to_stdout
        self.assert_returncode          = assert_returncode
        # self.returncode   = 0
        self.compatible_output  = compatible_output
        self.decode_output      = decode_output

        # Generate the args for subprocess.Popen
        args = self.cmd.split(' ', maxsplit=1)
        if args[0] in Config.bin_files:
            args[0]     = os.path.join(Config.bin_dir, args[0])
        args            = ' '.join(args)
        if Config.system_name == 'Windows':
            self.args   = args.replace('\'', '\"')
        else:
            self.args   = shlex.split(args, posix=os.name == 'posix')
            
        if len(Config.valgrind)>0:
            self.args = [ Config.valgrind ] + self.args 

        # Check stdin
        if self.stdin:
            if not isinstance(stdin, bytes):
                self.stdin = str(stdin).encode(self.encoding)

        self.run()

    def run(self):
        # Check stdout
        if self.redirect_stderr_to_stdout:
            stderr   = subprocess.STDOUT
        else:
            stderr   = subprocess.PIPE

        # Execute the command in subprocess
        try:
            with subprocess.Popen(self.args, cwd=self.cwd, env=self.env,
                                  stdin=subprocess.PIPE, stdout=subprocess.PIPE,
                                  stderr=stderr) as self.subprocess:
                try:
                    output  = self.subprocess.communicate(self.stdin, timeout=10)  # Assign (stdout, stderr) to output
                except subprocess.TimeoutExpired:
                    self.subprocess.kill()
                    output  = self.subprocess.communicate()
        except:
            raise RuntimeError('Failed to execute: {}'.format(self.args))
        output          = [i or b'' for i in output]
        output          = [i.rstrip(b'\r\n').rstrip(b'\n') for i in output] # Remove the last line break of the output

        # Extract stdout and stderr
        if self.compatible_output:
            output      = [i.replace(b'\r\n', b'\n')    for i in output]   # Fix dos line-endings
            output      = [i.replace(b'\\', rb'/')      for i in output]   # Fix dos path separators
        if self.decode_output:
            output      = [i.decode(self.encoding)      for i in output]
        self.stdout, self.stderr = [i or None           for i in output]

        # Check return code
        self.returncode = self.subprocess.returncode
        if self.assert_returncode and self.returncode not in self.assert_returncode:
            log.error('Failed to execute: {}'.format(self.args))
            log.error('The asserted return code is {}, but got {}'.format(str(self.assert_returncode), self.subprocess.returncode))
            log.info('OUTPUT:\n{}'.format(output[0] + output[1]))
            raise RuntimeError('\n' + log.to_str())


class Output:
    """
    Simulate the stdout buffer.
    You can use `out+=x` to simulate `print(x)`

    Sample:
    >>> out = Output()
    >>> out
    >>> str(out)
    >>> out += 'Hello'
    >>> out += 1
    >>> out += ['Hi' , 2]
    >>> out += None         # no effect
    >>> str(out)
    """
    def __init__(self):
        self.lines = []
        self.newline = '\n'

    def __str__(self):
        return self.newline.join(self.lines)

    # Comment it so that log does not automatically convert to str type
    # def __repr__(self):
    #     return str(self)

    def __add__(self, other):
        if isinstance(other, Executer):
            other = other.stdout
        if other != None:
            self.lines.append(str(other))
        return self

    def __radd__(self, other):
        return self.__add__(other)


def reportTest(testname, output: str, encoding=None,forgive=False):
    """ If the output of the test case is correct, this function returns None. Otherwise print its error. """
    output               = str(output) + '\n'
    encoding             = encoding or Config.encoding
    reference_file       = os.path.join(Config.data_dir, '{}.out'.format(testname))
    reference_output     = cat(reference_file, encoding=encoding)
    if reference_output == output:
        return
    log.error('The output of the testcase mismatch the reference')
    output_file = os.path.join(Config.tmp_dir, '{}.out'.format(testname))
    save(output, output_file, encoding=encoding)
    log.info('The output has been saved to file {}'.format(output_file))
    log.info('simply_diff:\n' + str(simply_diff(reference_file, output_file, encoding=encoding)))
    if forgive:
        print('Forgive: simply_diff:\n' + str(simply_diff(reference_file, output_file, encoding=encoding)))
    else:
        raise RuntimeError('\n' + log.to_str())


def ioTest(filename):
    src     = os.path.join(Config.data_dir, filename)
    out1    = os.path.join(Config.tmp_dir, '{}.1'.format(filename))
    out2    = os.path.join(Config.tmp_dir, '{}.2'.format(filename))
    Executer('iotest {src} {out1} {out2}', vars())
    assert md5sum(src) == md5sum(out1), 'The output file is different'
    assert md5sum(src) == md5sum(out2), 'The output file is different'


def eraseTest(filename):
    test_file   = filename + '.etst'
    good_file   = os.path.join(Config.data_dir, filename + '.egd')
    copyTestFile(filename, test_file)
    Executer('metacopy {test_file} {test_file}', vars())
    return diffCheck(good_file, test_file, in_bytes=True)


def copyTest(num, src, good):
    test_file   = '{}.c{}tst'.format(good, num)
    src_file    = os.path.join(Config.data_dir, src)
    good_file   = os.path.join(Config.data_dir, '{}.c{}gd'.format(good, num))
    copyTestFile(good, test_file)
    Executer('metacopy -a {src_file} {test_file}', vars())
    return diffCheck(good_file, test_file, in_bytes=True)


def iptcTest(num, src, good):
    test_file   = '{}.i{}tst'.format(good, num)
    src_file    = os.path.join(Config.data_dir, src)
    good_file   = os.path.join(Config.data_dir, '{}.i{}gd'.format(good, num))
    copyTestFile(good, test_file)
    Executer('metacopy -ip {src_file} {test_file}', vars())
    return diffCheck(good_file, test_file, in_bytes=True)


def printTest(filename):
    test_file   = filename + '.iptst'
    src_file    = os.path.join(Config.data_dir, filename)
    good_file   = os.path.join(Config.data_dir, filename + '.ipgd')
    copyTestFile(filename, test_file)

    e           = Executer('iptcprint {src_file}', vars(), assert_returncode=None, decode_output=False)
    stdout      = e.stdout.replace(Config.data_dir.replace(os.path.sep, '/').encode(), b'../data') # Ignore the difference of data_dir on Windows
    save(stdout + b'\n', test_file)

    return diffCheck(good_file, test_file, in_bytes=True)


def removeTest(filename):
    tmp         = 'temp'
    test_file   = filename + '.irtst'
    src_file    = os.path.join(Config.data_dir, filename)
    good_file   = os.path.join(Config.data_dir, filename + '.irgd')
    copyTestFile(filename, tmp)
    stdin       = """
r Iptc.Application2.Byline
r Iptc.Application2.Caption
r Iptc.Application2.Keywords
r Iptc.Application2.Keywords
r Iptc.Application2.Keywords
r Iptc.Application2.CountryName
""".lstrip('\n').encode()
    Executer('iptctest {tmp}', vars(), stdin=stdin)
    e           = Executer('iptcprint {tmp}', vars(), assert_returncode=None, decode_output=False)
    save(e.stdout + b'\n', test_file)
    return diffCheck(good_file, test_file, in_bytes=True)


def addModTest(filename):
    tmp         = 'temp'
    test_file   = filename + '.iatst'
    src_file    = os.path.join(Config.data_dir, filename)
    good_file   = os.path.join(Config.data_dir, filename + '.iagd')
    copyTestFile(filename, tmp)
    stdin       = """
a Iptc.Application2.Headline		  The headline I am
a Iptc.Application2.Keywords		  Yet another keyword
m Iptc.Application2.DateCreated		  2004-8-3
a Iptc.Application2.Urgency			  3
m Iptc.Application2.SuppCategory	  "bla bla ba"
a Iptc.Envelope.ModelVersion		  2
a Iptc.Envelope.TimeSent			  14:41:0-05:00
a Iptc.Application2.RasterizedCaption 230 42 34 2 90 84 23 146
""".lstrip('\n').encode()
    Executer('iptctest {tmp}', vars(), stdin=stdin)
    e           = Executer('iptcprint {tmp}', vars(), assert_returncode=None, decode_output=False)
    save(e.stdout + b'\n', test_file)
    return diffCheck(good_file, test_file, in_bytes=True)


def extendedTest(filename):
    tmp         = 'temp'
    test_file   = filename + '.ixtst'
    src_file    = os.path.join(Config.data_dir, filename)
    good_file   = os.path.join(Config.data_dir, filename + '.ixgd')
    copyTestFile(filename, tmp)
    stdin       = cat(os.path.join(Config.data_dir, 'ext.dat'))
    Executer('iptctest {tmp}', vars(), stdin=stdin)
    e           = Executer('iptcprint {tmp}', vars(), decode_output=False)
    save(e.stdout + b'\n', test_file)
    return diffCheck(good_file, test_file, in_bytes=True)


def runTestCase(num, img):
    """ Run the requested test case number with the given image """
    out_img     = 'test{}.jpg'.format(num)
    thumb_jpg   = 'thumb{}.jpg'.format(num)
    thumb_tif   = 'thumb{}.tif'.format(num)
    rm(out_img, thumb_jpg, thumb_tif)
    rm('iii', 'ttt')
    cp(img, out_img)
    out  = Output()
    out += '------------------------------------------------------------'

    e    = Executer('exifprint {img}', vars(), redirect_stderr_to_stdout=False, decode_output=False)
    out += e.stderr.decode() if e.stderr else None
    save(e.stdout, 'iii')

    e    = Executer('write-test {img} {num}', vars(), redirect_stderr_to_stdout=False, decode_output=False)
    out += e.stderr.decode() if e.stderr else None
    save(e.stdout, 'ttt')

    out += diff('iii', 'ttt')
    return str(out)

def runTest(cmd,raw=False):
    """ Executes a command in the shell. """

    if sys.platform == 'win32':
        args = cmd
    else:
        args = shlex.split(cmd)

    # Update PATH, LD_LIBRARY_PATH and DYLD_LIBRARY_PATH
    key = "PATH"
    bin_dir = os.path.dirname(system_tests.exiv2)
    if key in os.environ:
        os.environ[key] = os.path.join(bin_dir, os.environ[key])
    else:
        os.environ[key] = bin_dir

    for key in ["LD_LIBRARY_PATH", "DYLD_LIBRARY_PATH"]:
        lib_dir = os.path.join(os.path.dirname(os.path.dirname(system_tests.exiv2)), 'lib')
        if key in os.environ:
            os.environ[key] = lib_dir + os.pathsep + os.environ[key]
        else:
            os.environ[key] = lib_dir

    # Execute the command
    if raw:
        subprocess.Popen(args)
        out=None
    else:
        try:
            p = subprocess.Popen(args, stdout=subprocess.PIPE, shell=False)
            stdout, stderr   = p.communicate()
            if p.returncode != 0:
                print('{} returncode = {}'.format(cmd, p.returncode))
            # Split the output by newline
            out = stdout.decode('utf-8').replace('\r', '').rstrip('\n').split('\n')
        except:
            print('** {} died **'.format(cmd))

    return out


def verbose_version(verbose=False):
    """ Get the key-value pairs of Exiv2 verbose version  """
    vv    = {}
    exiv2=system_tests.exiv2
    lines = runTest(exiv2 + ' --verbose --version')
    for line in lines:
        kv = line.rstrip().split('=')
        if len(kv)  == 2:
            key, val = kv
            if not key in vv:
                vv[key] = val
            elif isinstance(vv[key], list):
                vv[key].append(val)
            else:
                vv[key] = [vv[key]]
    if verbose:
        for key in vv:
            val = vv[key]
            if isinstance(val, list):
                val = '[ {}   +{} ]'.format(val[0], len(val) - 1)
            print(key.ljust(20), val)
    return vv



# -*- coding: utf-8 -*-

import system_tests


class TestFirstPoC(metaclass=system_tests.CaseMeta):
    """
    Regression test for the two bugs described in:
    https://github.com/Exiv2/exiv2/issues/159

    We do not actually check the output of these files, we only check that we
    don't get a crash (the metadata are bogus anyway, so no point in checking).
    """
    url = "https://github.com/Exiv2/exiv2/issues/159"

    filename = [
        system_tests.path("$data_path/printStructure"),
        system_tests.path("$data_path/printStructure2")
    ]
    commands = [
        "$exiv2 " + filename[0],
        "$exiv2 -pS " + filename[1],
    ]
    retval = [0, 1]

    compare_stderr = system_tests.check_no_ASAN_UBSAN_errors
    compare_stdout = system_tests.check_no_ASAN_UBSAN_errors



# -*- coding: utf-8 -*-

import system_tests


@system_tests.CopyFiles("$data_path/exiv2-empty.jpg")
class EasyaccessTest(metaclass=system_tests.CaseMeta):

    filename = "$data_path/exiv2-empty_copy.jpg"

    commands = [
        """$exiv2 -u -v -M"set Exif.Image.Make Samsung" $filename""",
        "$easyaccess_test $filename"
    ]
    stdout = [
        """File 1/1: $filename
Set Exif.Image.Make "Samsung" (Ascii)
""",
         """Orientation           (                                   ) : 
ISO speed             (                                   ) : 
Date & time original  (                                   ) : 
Flash bias            (                                   ) : 
Exposure mode         (                                   ) : 
Scene mode            (                                   ) : 
Macro mode            (                                   ) : 
Image quality         (                                   ) : 
White balance         (                                   ) : 
Lens name             (                                   ) : 
Saturation            (                                   ) : 
Sharpness             (                                   ) : 
Contrast              (                                   ) : 
Scene capture type    (                                   ) : 
Metering mode         (                                   ) : 
Camera make           (Exif.Image.Make                    ) : Samsung
Camera model          (                                   ) : 
Exposure time         (                                   ) : 
FNumber               (                                   ) : 
Shutter speed value   (                                   ) : 
Aperture value        (                                   ) : 
Brightness value      (                                   ) : 
Exposure bias         (                                   ) : 
Max aperture value    (                                   ) : 
Subject distance      (                                   ) : 
Light source          (                                   ) : 
Flash                 (                                   ) : 
Camera serial number  (                                   ) : 
Focal length          (                                   ) : 
Subject location/area (                                   ) : 
Flash energy          (                                   ) : 
Exposure index        (                                   ) : 
Sensing method        (                                   ) : 
AF point              (                                   ) : 
"""
    ]
    stderr = [""] * 2
    retval = [0] * 2



# -*- coding: utf-8 -*-

from system_tests import CaseMeta, path, check_no_ASAN_UBSAN_errors

class XMPUtilsSetTimeZoneIntegerOverflow(metaclass=CaseMeta):
    """
    Regression test for the bug described in:
    https://github.com/Exiv2/exiv2/issues/1901
    """
    url = "https://github.com/Exiv2/exiv2/issues/1901"

    filename1 = path("$data_path/issue_1901_poc1.xmp")
    filename2 = path("$data_path/issue_1901_poc2.xmp")
    filename3 = path("$data_path/issue_1901_poc3.xmp")
    filename4 = path("$data_path/issue_1901_poc4.xmp")
    commands = ["$exiv2 -q $filename1",
                "$exiv2 -q $filename2",
                "$exiv2 -q $filename3",
                "$exiv2 -q $filename4"]
    stderr = ["""$filename1: No Exif data found in the file
""",
              """$filename2: No Exif data found in the file
""",
              """$filename3: No Exif data found in the file
""",
              ""]
    retval = [253, 253, 253, 0]

    compare_stdout = check_no_ASAN_UBSAN_errors
